/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include "deleteuserdialog.h"
#include "containerutils.h"
#include "Loqqer.h"

#include <QtGui>

DeleteUserDialog::DeleteUserDialog( QWidget* parent ) :
    QDialog( parent )
{
    ui.setupUi(this);

    // Want etched, not flat
    ui.line->setFrameShadow( QFrame::Sunken );

    SettingsServiceInterface* settings = settingsService();

    // Fill drop down with users
    foreach(QString user, settings->allUsers())
    {
        ui.userCombo->addItem(user);
    }
}

void
DeleteUserDialog::accept()
{
    if ( ui.userCombo->count() <= 1 )
    {
        QMessageBox::critical( this, tr( "Error" ),
            tr( "There must be at least one user in the system." ),
            QMessageBox::Ok, QMessageBox::NoButton );
        return;
    }

    SettingsServiceInterface* settings = settingsService();
    QString userToDelete = ui.userCombo->currentText();

    if ( userToDelete == settings->currentUsername() )
    {
        QMessageBox::critical( this, tr( "Error" ),
            tr( "You can't delete the currently active user." ),
            QMessageBox::Ok, QMessageBox::NoButton );
        return;
    }

    int confirm = QMessageBox::question( this, tr( "Confirm" ),
        tr( "Do you really want to delete user %1?" ).arg( userToDelete ),
        QMessageBox::Yes, QMessageBox::No );

    if ( confirm == QMessageBox::Yes )
    {
        settings->deleteUser( userToDelete );
        QDialog::accept();
    }
}
