/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef EXCEPTIONS_H
#define EXCEPTIONS_H

#include <QString>
#include <QCoreApplication>
#include <QtGlobal>

/*************************************************************************/ /**
    Exception base class
******************************************************************************/
class QException
{
public:
    QException(const char* msg) {
        m_what = msg;
    }
    QString what() const { return m_what; }
    QString tr_what() const {
        return QCoreApplication::translate("Exception", qPrintable(m_what));
    }

private:
    QString m_what;
};

/*************************************************************************/ /**
    Internet connection errors
******************************************************************************/
class ConnectionException : public QException
{
public:
    ConnectionException(const char* msg) : QException(msg) { }
};

/*************************************************************************/ /**
    General networking errors
******************************************************************************/
class NetworkException : public QException
{
public:
    NetworkException(const char* msg) : QException(msg) { }
};

/*************************************************************************/ /**
    Used if an invalid player plugin submits tracks
******************************************************************************/
class BadClientException : public QException
{
public:
    BadClientException(const char* msg) : QException(msg) { }
};

/*************************************************************************/ /**
    Used if we failed parsing a player command
******************************************************************************/
class ParseException : public QException
{
public:
    ParseException(const char* msg) : QException(msg) { }
};

/*************************************************************************/ /**
    Used if we receive an invalid command from a player plugin
******************************************************************************/
class BadCommandException : public QException
{
public:
    BadCommandException(const char* msg) : QException(msg) { }
};

#endif // EXCEPTIONS_H
