/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include "failedlogindialog.h"
#include "containerutils.h"
#include "Loqqer.h"

#include <QtGui>

FailedLoginDialog::FailedLoginDialog( QWidget* parent ) :
    QDialog( parent )
{
    LOGL( 3, "Launching FailedLoginDialog" );

    ui.setupUi(this);

    // Want etched, not flat
    ui.line->setFrameShadow( QFrame::Sunken );
    ui.proxyLine->setFrameShadow( QFrame::Sunken );

    QPixmap cross( dataPath( "icons/qt_cross.png" ) );
    ui.imgLabel->setPixmap( cross );
    ui.errorLabel->setText(
        tr("Couldn't connect to the internet to verify your user details.\n\n"
           "If you use a proxy to connect to the internet, please click\nthe "
           "button below to enter your proxy details."));

    // Remove and hide the proxy box, and display it when the button is clicked
    ui.proxyFrame->hide();
    ui.proxyLine->hide();
    layout()->removeItem( ui.hboxLayout2 );
    adjustSize();

    // Populate proxy settings
    SettingsServiceInterface* settings = settingsService();
    ui.proxyHostEdit->setText( settings->proxyHost() );
    ui.proxyPortEdit->setText( QString::number(settings->proxyPort()) );
    ui.proxyUsernameEdit->setText( settings->proxyUsername() );
    ui.proxyPasswordEdit->setText( settings->proxyPassword() );

    connect(ui.showProxyButton, SIGNAL(clicked()),
            this,               SLOT  (showProxyFrame()));
}

void
FailedLoginDialog::showProxyFrame()
{
    if (ui.proxyFrame->isHidden())
    {
        ui.vboxLayout->insertLayout( 3, ui.hboxLayout2 );
        ui.proxyLine->show();
        ui.proxyFrame->show();
        ui.showProxyButton->setEnabled( false );
        adjustSize();
    }
}

void
FailedLoginDialog::accept()
{
    LOGL(4, "FailedLoginDialog::accept()");

    if (!ui.proxyFrame->isHidden())
    {
        // Save out proxy settings
        SettingsServiceInterface* settings = settingsService();
        settings->setProxyHost(ui.proxyHostEdit->text());
        settings->setProxyPort(ui.proxyPortEdit->text().toInt());
        settings->setProxyUsername(ui.proxyUsernameEdit->text());
        settings->setProxyPassword(ui.proxyPasswordEdit->text());
        
        settings->setUseProxy(!settings->proxyHost().isEmpty());

        settings->save();
    }

    LOGL(4, "FailedLoginDialog calling QDialog::accept()");

    QDialog::accept();
}
