/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef HTTPINPUT_H
#define HTTPINPUT_H

#include <metadata.h>
#include <inputinterface.h>
#include <webserviceinterface.h>
#include "playercommands.h"

#include "http.h"
#include <QMutex>
#include <QObject>
#include <QUrl>

#define MAX_HTTP_BUFFER 98304

class HTTPInput : public InputInterface
{
    Q_OBJECT
    Q_INTERFACES( InputInterface )

    public:
        HTTPInput();

        bool isStreaming() { return m_streaming; }

        int bufferSize() { return m_bufferSize; }
        void setBufferSize( int size ) { m_bufferSize = ( size > MAX_HTTP_BUFFER ) ? MAX_HTTP_BUFFER : size; }

    public slots:
        void startStreaming();
        void stopStreaming();

        void load( const QString& url );
        void nextSong() { m_skipToSync = true; m_iWebService->skip(); }

        void requestData();

    signals:
        void requestedStreaming();
        void preparedStreaming();
        void startedStreaming();
        void stoppedStreaming();

        void error( int errorCode, const QString& reason );
        void bufferFull();
        void bufferEmpty( int currentSize = 0 );
        void clearBuffers();

        void newData( const QByteArray& buffer );
        void newMetaData( const MetaData& metaData );

        // Temporary bodge signal to notify player listener of new track
        void listenerCommand( const TrackInfo& trackInfo, EPlayerCommand cmd );

    private:
        QUrl m_url;
        Http m_http;
        QByteArray m_buffer;
        QMutex m_mutex;

        bool m_streaming;
        bool m_skipToSync;
        bool m_skippedData;
        bool m_refillBuffer;
        int m_bufferSize;

        WebServiceInterface *m_iWebService;

    private slots:
        void setStreamingUrl( const QUrl &url ) {
            m_url = url;
        }
        void setMetaData( MetaData metaData );
        void stationChanged();
        void stationChangeFailed();

        void dataAvailable( const QHttpResponseHeader &resp );
        void responseHeaderReceived( const QHttpResponseHeader &resp );
        void stateChanged( int state );
};

#endif
