/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include "iconshack.h"
#include "Loqqer.h"

using namespace std;

// Singleton instance needs to be initialised
IconShack* IconShack::mpInstance = NULL;

/******************************************************************************
    instance
******************************************************************************/
IconShack&
IconShack::instance()
{
    if ( !mpInstance )
    {
        mpInstance = new IconShack();
    }

    return *mpInstance;
}

/******************************************************************************
    GetGoodUserIcon
******************************************************************************/
QPixmap
IconShack::GetGoodUserIcon(
    UserIconColour eColour)
{
    if (!mbIconsLoaded)
    {
        LoadIcons();
    }

    if (maGoodIcons[eColour].isNull())
    {
        return QPixmap(16, 16);
    }
    else
    {
        return maGoodIcons[eColour];
    }

}

/******************************************************************************
    GetGoodUserIconCollapsed
******************************************************************************/
QPixmap
IconShack::GetGoodUserIconCollapsed(
    UserIconColour eColour)
{
    if (!mbIconsLoaded)
    {
        LoadIcons();
    }

    if (maGoodIconsCollapsed[eColour].isNull())
    {
        return QPixmap(32, 32);
    }
    else
    {
        return maGoodIconsCollapsed[eColour];
    }

}

/******************************************************************************
    GetGoodUserIconExpanded
******************************************************************************/
QPixmap
IconShack::GetGoodUserIconExpanded(
    UserIconColour eColour)
{
    if (!mbIconsLoaded)
    {
        LoadIcons();
    }

    if (maGoodIconsExpanded[eColour].isNull())
    {
        return QPixmap(32, 32);
    }
    else
    {
        return maGoodIconsExpanded[eColour];
    }

}

/******************************************************************************
    GetExclUserIcon
******************************************************************************/
QPixmap
IconShack::GetExclUserIcon(
    UserIconColour eColour)
{
    if (!mbIconsLoaded)
    {
        LoadIcons();
    }

    if (maExclIcons[eColour].isNull())
    {
        return QPixmap(16, 16);
    }
    else
    {
        return maExclIcons[eColour];
    }
}    

/******************************************************************************
    GetDisabledUserIcon
******************************************************************************/
QPixmap
IconShack::GetDisabledUserIcon(
    UserIconColour eColour)
{
    if (!mbIconsLoaded)
    {
        LoadIcons();
    }
    
    if (maDisabledIcons[eColour].isNull())
    {
        return QPixmap(16, 16);
    }
    else
    {
        return maDisabledIcons[eColour];
    }
}    

/******************************************************************************
    LoadIcons
******************************************************************************/
void
IconShack::LoadIcons()
{
    maGoodIcons[eRed].load("data/icons/user_red.png");
    maGoodIcons[eBlue].load("data/icons/user_blue.png");
    maGoodIcons[eGreen].load("data/icons/user_green.png");
    maGoodIcons[eOrange].load("data/icons/user_orange.png");
    maGoodIcons[eBlack].load("data/icons/user_black.png");

    maGoodIconsCollapsed[eRed].load("data/icons/user_red32_collapse.png");
    maGoodIconsCollapsed[eBlue].load("data/icons/user_blue32_collapse.png");
    maGoodIconsCollapsed[eGreen].load("data/icons/user_green32_collapse.png");
    maGoodIconsCollapsed[eOrange].load("data/icons/user_orange32_collapse.png");
    maGoodIconsCollapsed[eBlack].load("data/icons/user_black32_collapse.png");
    
    maGoodIconsExpanded[eRed].load("data/icons/user_red32_expand.png");
    maGoodIconsExpanded[eBlue].load("data/icons/user_blue32_expand.png");
    maGoodIconsExpanded[eGreen].load("data/icons/user_green32_expand.png");
    maGoodIconsExpanded[eOrange].load("data/icons/user_orange32_expand.png");
    maGoodIconsExpanded[eBlack].load("data/icons/user_black32_expand.png");

    maExclIcons[eRed].load("data/icons/user_red_caching.png");
    maExclIcons[eBlue].load("data/icons/user_blue_caching.png");
    maExclIcons[eGreen].load("data/icons/user_green_caching.png");
    maExclIcons[eOrange].load("data/icons/user_orange_caching.png");
    maExclIcons[eBlack].load("data/icons/user_black_caching.png");

    maDisabledIcons[eRed].load("data/icons/user_disabled.png");
    maDisabledIcons[eBlue].load("data/icons/user_disabled.png");
    maDisabledIcons[eGreen].load("data/icons/user_disabled.png");
    maDisabledIcons[eOrange].load("data/icons/user_disabled.png");
    maDisabledIcons[eBlack].load("data/icons/user_disabled.png");

    mbIconsLoaded = true;

    LOG(4, "Icons loaded\n");
}
