/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef INPUT_INTERFACE_H
#define INPUT_INTERFACE_H

#include <QObject>

#include "metadata.h"

class InputInterface : public QObject
{
    public:
        virtual ~InputInterface() {}

        virtual bool isStreaming() = 0;

        virtual int bufferSize() = 0;
        virtual void setBufferSize( int size ) = 0;

    public slots:
        virtual void preparedStreaming() = 0;
        virtual void startStreaming() = 0;
        virtual void stopStreaming() = 0;

        virtual void load( const QString& url ) = 0;
        virtual void nextSong() = 0;

        virtual void requestData() = 0;

    signals:
        virtual void startedStreaming() = 0;
        virtual void stoppedStreaming() = 0;

        virtual void error( int errorCode, const QString& reason ) = 0;

        virtual void bufferFull() = 0;
        virtual void bufferEmpty( int currentSize = 0 ) = 0;
        virtual void clearBuffers() = 0;

        virtual void newData( const QByteArray& buffer ) = 0;
        virtual void newMetaData( const MetaData& metaData ) = 0;
};

Q_DECLARE_INTERFACE( InputInterface, "fm.last.Input/1.0" )

#endif
