/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include "itunesscript.h"

#ifdef Q_WS_MAC

#include "containerutils.h"
#include "playerlistener.h"
#include "Loqqer.h"

#include <QDebug>
#include <QTcpSocket>
#include <QHostAddress>

ComponentInstance theComponent = NULL;

OSAID m_OSA_CheckITunes = kOSANullScript;
OSAID m_OSA_CheckResponse = kOSANullScript;
OSAID m_OSA_CheckPlaying = kOSANullScript;
OSAID m_OSA_CheckPaused = kOSANullScript;
OSAID m_OSA_CheckArtist = kOSANullScript;
OSAID m_OSA_CheckAlbum = kOSANullScript;
OSAID m_OSA_CheckTrack = kOSANullScript;
OSAID m_OSA_CheckDuration = kOSANullScript;
OSAID m_OSA_CheckPath = kOSANullScript;


bool ITunesScript::m_paused = false;
QString ITunesScript::m_lastHash;


static
QString CFStringToQString( CFStringRef s )
{
    QString result;

    if ( s != NULL )
    {
        CFIndex length = 1 + 4 * CFStringGetLength( s ); // Can be 32 bit unicode, hence 4 times + 8 bits for the null
        char* buffer = new char[length];

        if (CFStringGetCString( s, buffer, length, kCFStringEncodingUTF8 ))
        {
            result = QString::fromUtf8(buffer);
        }
        else
        {
            qWarning() << "itunesplayer.cpp: CFString conversion failed.";
        }

        delete buffer;
    }

    return result;
}


ITunesScript::ITunesScript( QObject* parent, const CPlayerListener& listener ) :
//     QThread( parent ),
    m_listener( listener )
{

    /* open the scripting component */
//     theComponent = OpenDefaultComponent( kOSAComponentType, typeAppleScript );
//     if ( theComponent )
//         LOG( 3, QString( "AppleScript available:" ) << AppleScriptAvailable() << "\n" );

    CFNotificationCenterRef center = CFNotificationCenterGetDistributedCenter();
    CFNotificationCenterAddObserver( center, this, ITunesScript::iTunesCallback, CFSTR( "com.apple.iTunes.playerInfo" ), NULL, CFNotificationSuspensionBehaviorDeliverImmediately );

//     if ( AppleScriptAvailable() )
//     {
//         m_timerData.setInterval( 3000 );
//         m_timer.setInterval( 1750 );
//         connect( &m_timer, SIGNAL( timeout() ), this, SLOT( checkITunes() ) );
//         connect( &m_timerData, SIGNAL( timeout() ), this, SLOT( ITunesData() ) );
//
//         m_timer.start();
//     }

//     start( QThread::NormalPriority );
}


void
ITunesScript::iTunesCallback( CFNotificationCenterRef, void* observer, CFStringRef, const void*, CFDictionaryRef info )
{
    ITunesScript* controller = (ITunesScript*) observer;

    CFStringRef cf_state = (CFStringRef) CFDictionaryGetValue( info, CFSTR( "Player State" ) );
    if ( CFStringCompare( cf_state, CFSTR( "Paused" ), 0 ) == kCFCompareEqualTo )
    {
        qDebug() << "Paused";

        m_paused = true;

        QString data = "PAUSE c=osx\n";
        controller->transmit( data );
    }
    else
    if ( CFStringCompare( cf_state, CFSTR( "Stopped" ), 0 ) == kCFCompareEqualTo )
    {
        qDebug() << "Stopped";

        m_paused = false;

        QString data = "STOP c=osx\n";
        controller->transmit( data );
    }
    else
    if ( CFStringCompare( cf_state, CFSTR( "Playing" ), 0 ) == kCFCompareEqualTo )
    {
        qDebug() << "Playing";

        QString const artist = CFStringToQString( (CFStringRef) CFDictionaryGetValue( info, CFSTR( "Artist" ) ) );
        QString const track = CFStringToQString( (CFStringRef) CFDictionaryGetValue( info, CFSTR( "Name" ) ) );
        QString const album = CFStringToQString( (CFStringRef) CFDictionaryGetValue( info, CFSTR( "Album" ) ) );
        QString const path = CFStringToQString( (CFStringRef) CFDictionaryGetValue( info, CFSTR( "Location" ) ) );

        qDebug() << "Artist:" << artist;
        qDebug() << "Name:" << track;
        qDebug() << "Album:" << album;
        
        CFNumberRef cf_time = (CFNumberRef) CFDictionaryGetValue( info, CFSTR( "Total Time" ) );
        int time = 0;
        if ( cf_time != NULL && !CFNumberGetValue( cf_time, kCFNumberIntType, &time ) )
        {
            qWarning() << "ITunesScript: Duration conversion failed.";
        }

        QString data = "START c=osx&a=" + controller->encodeAmp( artist ) +
                        "&t=" + controller->encodeAmp( track ) +
                        "&b=" + controller->encodeAmp( album ) +
                        "&l=" + QString::number( (unsigned int)time / 1000 ) +
                        "&p=" + controller->encodeAmp( path ) + '\n';

        if ( m_lastHash == MD5Digest( data.toUtf8() ) )
        {
            if ( m_paused )
            {
                m_paused = false;

                QString const data = "RESUME c=osx\n";
                controller->transmit( data );
            }

            return;
        }

        m_paused = false;
        m_lastHash = MD5Digest( data.toUtf8() );
        controller->transmit( data );

//           tune.setTime( (unsigned int) ( time / 1000 ) );
//           emit controller->playing( tune );
    }
    else
    {
          qWarning( "ITunesScript: Unknown state." );
    }
}


void
ITunesScript::checkITunes()
{
//     qDebug() << "Check ITunes state";
// 
//     if ( isITunesRunning() )
//     {
//         if ( !isITunesResponding() )
//             return;
// 
//         bool playing = isITunesPlaying();
// 
//         //qDebug() << "m_paused: " << m_paused << ", playing: " << playing <<
//         //"isITunesPaused(): " << isITunesPaused() << ", m_playing: " << m_playing << ", timer active: " << m_timerData.isActive();
// 
//         if ( playing && m_paused )
//         {
//             m_paused = false;
//             QString data = "RESUME c=osx\n";
//             transmit( data );
//         }
//         else
//         if ( !m_paused && isITunesPaused() )
//         {
//             m_paused = true;
//             QString data = "PAUSE c=osx\n";
//             transmit( data );
//         }
// 
//         if ( !playing && !m_paused && m_playing )
//         {
//             m_timerData.stop();
//             m_playing = false;
//             m_lastHash = "";
// 
//             QString data = "STOP c=osx\n";
//             transmit( data );
//         }
// 
//         m_playing = playing || m_paused;
// 
//         if ( !m_timerData.isActive() && playing )
//         {
//             m_timerData.start();
//             ITunesData();
//         }
//     }
//     else
//     if ( m_playing || m_paused )
//     {
//         m_timerData.stop();
//         m_playing = false;
//         m_paused = false;
//         m_lastHash = "";
// 
//         QString data = "STOP c=osx\n";
//         transmit( data );
//     }

}


void
ITunesScript::ITunesData()
{
    qDebug() << "Check ITunes song";

    if ( isITunesRunning() )
    {
        if ( !isITunesResponding() )
            return;

        QString data = "START c=osx&a=" + encodeAmp( artist() ) +
                        "&t=" + encodeAmp( track() ) +
                        "&b=" + encodeAmp( album() ) +
                        "&l=" + duration() +
                        "&p=" + encodeAmp( path() ) + "\n";

        if ( m_lastHash == MD5Digest( data.toUtf8() ) )
            return;

        m_lastHash = MD5Digest( data.toUtf8() );
        transmit( data );

        m_paused = false;
    }

}


void
ITunesScript::transmit( QString data )
{
//     qDebug() << "Gonna transmit now";

    QTcpSocket socket;
    int port = m_listener.GetPort();

    LOGL( 3, "ITunesScript data being sent: " << data << " to port: " << port );

    socket.connectToHost( QHostAddress::LocalHost, port );
    if ( socket.waitForConnected( 1000 ) )
    {
        int bytesWritten = socket.write( data.toUtf8() );
        if ( bytesWritten == -1 )
        {
            LOGL( 1, "Sending submission through socket failed." )
        }
        socket.flush();
        socket.waitForDisconnected( 1000 );
    }
}


bool
ITunesScript::isITunesResponding()
{
    if ( m_OSA_CheckResponse == kOSANullScript )
    {
        QString script =
            "set itunes_active to false\n"
            "with timeout of 1 seconds\n"
            "tell application \"iTunes\"\n"
                "if player state is playing then set itunes_active to true\n"
            "end tell\n"
            "end timeout\n"
            "return itunes_active\n";

        m_OSA_CheckResponse = LowCompileAppleScript( script.toUtf8(), script.toUtf8().length() );
    }

    QString r;
    if ( LowExecAppleScript( m_OSA_CheckResponse, r ) )
    {
        qDebug() << "iTunes responding";
        return true;
    }
    else
    {
        qDebug() << "iTunes not responding";
        return false;
    }
}


bool
ITunesScript::isITunesRunning()
{
    if ( m_OSA_CheckITunes == kOSANullScript )
    {
        QString script =
            "set itunes_active to false\n"
            "with timeout of 1 seconds\n"
            "tell application \"Finder\"\n"
                "if (get name of every process) contains \"iTunes\" then set itunes_active to true\n"
            "end tell\n"
            "end timeout\n"
            "return itunes_active\n";

        m_OSA_CheckITunes = LowCompileAppleScript( script.toUtf8(), script.toUtf8().length() );
    }

    QString r;
    LowExecAppleScript( m_OSA_CheckITunes, r );

    return ( r == "true" );
}


bool
ITunesScript::isITunesPlaying()
{
    if ( !isITunesRunning() )
        return false;

    if ( m_OSA_CheckPlaying == kOSANullScript )
    {
        QString script =
            "set itunes_active to false\n"
            "with timeout of 1 seconds\n"
            "tell application \"iTunes\"\n"
                "if player state is playing then set itunes_active to true\n"
            "end tell\n"
            "end timeout\n"
            "return itunes_active\n";

        m_OSA_CheckPlaying = LowCompileAppleScript( script.toUtf8(), script.toUtf8().length() );
    }

    QString r;
    LowExecAppleScript( m_OSA_CheckPlaying, r );

    return ( r == "true" );
}


bool
ITunesScript::isITunesPaused()
{
    if ( !isITunesRunning() )
        return false;

    if ( m_OSA_CheckPaused == kOSANullScript )
    {
        QString script =
            "set itunes_active to false\n"
            "with timeout of 1 seconds\n"
            "tell application \"iTunes\"\n"
                "if player state is paused then set itunes_active to true\n"
            "end tell\n"
            "end timeout\n"
            "return itunes_active\n";

        m_OSA_CheckPaused = LowCompileAppleScript( script.toUtf8(), script.toUtf8().length() );
    }

    QString r;
    LowExecAppleScript( m_OSA_CheckPaused, r );

    return ( r == "true" );
}


QString
ITunesScript::artist()
{
//     qDebug() << "Getting artist";

    if ( !isITunesRunning() )
        return QString();

    if ( m_OSA_CheckArtist == kOSANullScript )
    {
        QString script =
            "set artistname to \"\"\n"
            "with timeout of 1 seconds\n"
            "tell application \"iTunes\"\n"
                "set artistname to artist of current track\n"
            "end tell\n"
            "end timeout\n"
            "return artistname\n";

        m_OSA_CheckArtist = LowCompileAppleScript( script.toUtf8(), script.toUtf8().length() );
    }

    QString r;
    LowExecAppleScript( m_OSA_CheckArtist, r );

    return r;
}


QString
ITunesScript::album()
{
//     qDebug() << "Getting album";

    if ( !isITunesRunning() )
        return QString();

    if ( m_OSA_CheckAlbum == kOSANullScript )
    {
        QString script =
            "set albumname to \"\"\n"
            "with timeout of 1 seconds\n"
            "tell application \"iTunes\"\n"
                "set albumname to album of current track\n"
            "end tell\n"
            "end timeout\n"
            "return albumname\n";

        m_OSA_CheckAlbum = LowCompileAppleScript( script.toUtf8(), script.toUtf8().length() );
    }

    QString r;
    LowExecAppleScript( m_OSA_CheckAlbum, r );

    return r;
}


QString
ITunesScript::track()
{
//     qDebug() << "Getting track";

    if ( !isITunesRunning() )
        return QString();

    if ( m_OSA_CheckTrack == kOSANullScript )
    {
        QString script =
            "set trackname to \"\"\n"
            "with timeout of 1 seconds\n"
            "tell application \"iTunes\"\n"
                "set trackname to name of current track\n"
            "end tell\n"
            "end timeout\n"
            "return trackname\n";

        m_OSA_CheckTrack = LowCompileAppleScript( script.toUtf8(), script.toUtf8().length() );
    }

    QString r;
    LowExecAppleScript( m_OSA_CheckTrack, r );

    return r;
}


QString
ITunesScript::duration()
{
//     qDebug() << "Getting duration";

    if ( !isITunesRunning() )
        return "-1";

    if ( m_OSA_CheckDuration == kOSANullScript )
    {
        QString script =
            "set token to \"\"\n"
            "with timeout of 1 seconds\n"
            "tell application \"iTunes\"\n"
                "set token to duration of current track\n"
            "end tell\n"
            "end timeout\n"
            "return token\n";

        m_OSA_CheckDuration = LowCompileAppleScript( script.toUtf8(), script.toUtf8().length() );
    }

    QString r;
    LowExecAppleScript( m_OSA_CheckDuration, r );

    if ( r.contains( "." ) )
        r = r.split( "." )[0];
    return r;
}


QString
ITunesScript::path()
{
//     qDebug() << "Getting path";

    if ( !isITunesRunning() )
        return QString();

    if ( m_OSA_CheckPath == kOSANullScript )
    {
        QString script =
            "set filepath to \"\"\n"
            "with timeout of 1 seconds\n"
            "tell application \"iTunes\"\n"
                "set filepath to location of current track\n"
            "end tell\n"
            "end timeout\n"
            "return filepath\n";

        m_OSA_CheckPath = LowCompileAppleScript( script.toUtf8(), script.toUtf8().length() );
    }

    QString r;
    LowExecAppleScript( m_OSA_CheckPath, r );

    return r;
}



/* AppleScriptAvailable returns true if AppleScript is available
and the routines defined herein can be called. */
bool AppleScriptAvailable()
{
//     qDebug() << "Getting available";

    long response;
    if ( Gestalt( gestaltAppleScriptAttr, &response ) != noErr )
        response = 0;

    return ( ( response & ( 1 << gestaltAppleScriptPresent ) ) != 0 );
}


OSAID LowCompileAppleScript( const void* text, long textLength )
{
//     qDebug() << "Compiling script";

    QString result;
    OSStatus err;
    AEDesc scriptTextDesc;
    OSAID scriptID = kOSANullScript;

    AECreateDesc( typeNull, NULL, 0, &scriptTextDesc );

    /* put the script text into an aedesc */
    err = AECreateDesc( typeChar, text, textLength, &scriptTextDesc );
    if ( err != noErr )
        goto bail;

    /* compile the script */
    err = OSACompile( theComponent, &scriptTextDesc, kOSAModeNull, &scriptID );
    if ( err != noErr )
        goto bail;

bail:
    return scriptID;
}


/* LowRunAppleScript compiles and runs an AppleScript
provided as text in the buffer pointed to by text.  textLength
bytes will be compiled from this buffer and run as an AppleScript
using all of the default environment and execution settings.  If
resultData is not NULL, then the result returned by the execution
command will be returned as typeChar in this descriptor record
(or typeNull if there is no result information).  If the function
returns errOSAScriptError, then resultData will be set to a
descriptive error message describing the error (if one is
available). */
bool LowExecAppleScript( OSAID scriptID, QString& resultToken )
{
//     qDebug() << "Executing script";

    QString s;
    char result[4096] = "\0";

    OSStatus err;
    AEDesc resultData;
    OSAID resultID = kOSANullScript;

//     qDebug() << "Running script";
    /* run the script/get the result */
    err = OSAExecute( theComponent, scriptID, kOSANullScript, kOSAModeAlwaysInteract, &resultID );

//     qDebug() << "Running script done" << err;
    if ( err == errOSAScriptError )
    {
        OSAScriptError( theComponent, kOSAErrorMessage, typeUTF8Text, &resultData );
        int length = AEGetDescDataSize( &resultData );
        {
            AEGetDescData( &resultData, result, length < 4096 ? length : 4096 );
            result[ length ] = '\0';
            s = QString::fromUtf8( (char *)&result, length );
            LOG( 1, "AppleScript error: " << s << "\n" );
        }

        return false;
    }
    else if ( err == noErr && resultID != kOSANullScript )
    {
//         qDebug() << "Getting script result";

        OSADisplay( theComponent, resultID, typeUTF8Text, kOSAModeNull, &resultData );

        int length = AEGetDescDataSize( &resultData );
        {
            AEGetDescData( &resultData, result, length < 4096 ? length : 4096 );
            s = QString::fromUtf8( (char*)&result, length );

            // Strip surrounding quotes
            if ( s.startsWith( "\"" ) && s.endsWith( "\"" ) )
            {
                s = s.mid( 1, s.length() - 2 );
            }

            // iTunes sometimes gives us strings with null terminators which
            // fucks things up if we don't remove them.
            while ( s.endsWith( QChar( QChar::Null ) ) )
            {
                s.chop( 1 );
            }

            // It also escapes quotes so convert those too
            s.replace( "\\\"", "\"" );
        }
    }
    else
    {
        // AppleScript not responding
        return false;
    }

//     qDebug() << "Executing script done";
    if ( resultID != kOSANullScript )
        OSADispose( theComponent, resultID );

    resultToken = s;
    return true;
}

#endif // end Q_WS_MAC
