/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include "confirmdialog.h"
#include "containerutils.h"
#include "Loqqer.h"

#include <QtGui>
#include <QString>
#include <QtDebug>

ConfirmDialog::ConfirmDialog(
    Operation op,
    QString text,
    QWidget *parent ) :
        QDialog( parent ),
        m_op( op )
{
    ui.setupUi(this);

    // Want etched, not flat
    ui.line->setFrameShadow( QFrame::Sunken );

    setText( text );

    switch ( m_op )
    {
        case Tag:
            setIcon( dataPath( "buttons/tag.png" ) );
            m_opString = "tag";
            break;

        case Recommend:
            setIcon( dataPath( "buttons/recommend.png" ) );
            m_opString = "recommend";
            break;

        case Love:
            setIcon( dataPath( "buttons/love.png" ) );
            m_opString = "love";
            break;

        case Ban:
            setIcon( dataPath( "buttons/ban.png" ) );
            m_opString = "ban";
            break;

        case Quit:
            setIcon( dataPath( "app_55.png" ) );
            m_opString = "quit";
            break;
    }

    connect( ui.yesButton, SIGNAL( clicked() ),
             this,         SLOT  ( accept() ) );
    connect( ui.noButton,  SIGNAL( clicked() ),
             this,         SLOT  ( reject() ) );
}

void
ConfirmDialog::setIcon(
    const QPixmap& icon)
{
    ui.iconLabel->setPixmap( icon );
}

void
ConfirmDialog::setIcon(
    const QString& file)
{
    ui.iconLabel->setPixmap(
        QPixmap( file ).scaled(
            32, 32, Qt::IgnoreAspectRatio, Qt::SmoothTransformation) );
}

void
ConfirmDialog::setText(
    const QString& text)
{
    ui.messageLabel->setText( text );
}

bool
ConfirmDialog::isDontAskChecked()
{
    return ui.dontAskCheck->checkState() == Qt::Checked;
}

int
ConfirmDialog::exec()
{
    SettingsServiceInterface* settings = settingsService();
    bool dontAsk = settings->dontAsk( m_opString );

    if ( dontAsk )
    {
        return QDialog::Accepted;
    }
    else
    {
        return QDialog::exec();
    }
}

void
ConfirmDialog::accept()
{
    saveCheckState();

    QDialog::accept();
}

void
ConfirmDialog::reject()
{
    saveCheckState();

    QDialog::reject();
}

void
ConfirmDialog::saveCheckState()
{
    SettingsServiceInterface* settings = settingsService();
    settings->setDontAsk( m_opString, isDontAskChecked() );
}
