/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef HTTP_H
#define HTTP_H

#include "common.h"
#include <QHttp>
#include <QHash>
#include <QStack>

class MCEXP Http : public QHttp
{
    Q_OBJECT

    public:
        Http( QObject* parent = 0 );
        Http( const QString& hostName, int port = 80, QObject* parent = 0 );

        int get( const QString& path, bool useCache = false );
        int post( const QString& path, QIODevice* data );
        int post( const QString& path, const QByteArray& data );
        int request( const QHttpRequestHeader& header, QIODevice* data = 0, QIODevice* to = 0 );
        int request( const QHttpRequestHeader& header, const QByteArray& data, QIODevice* to = 0, bool useCache = false );

        int setHost( const QString & hostName, quint16 port = 80 ) { m_hostname = hostName; return QHttp::setHost( hostName, port ); }
        QByteArray readAll() { checkBuffer(); QByteArray b = m_buffer; m_buffer.clear(); return b; }

        virtual qint64 bytesAvailable() { return m_buffer.isEmpty() ? QHttp::bytesAvailable() : m_buffer.size(); }
        QString pathToCachedCopy( QString cacheKey ) const;

    signals:
        void errorOccured( int error, const QString& errorString );
        void dataAvailable( const QByteArray& buffer );

    private slots:
        void dataFinished( int id, bool error );
        void getFromCache();

    private:
        class CachedRequestData
        {
            public:
                int m_id;
                QString m_cacheKey;

                CachedRequestData() : m_id( -1 ), m_cacheKey( "" ) { }
                CachedRequestData( int id, QString cacheKey ) :
                    m_id( id ), m_cacheKey( cacheKey ) { }
        };

        int m_dataID;
        QByteArray m_buffer;
        QString m_cachePath;
        QString m_hostname;

        // Used for cached metadata requests
        QHash<int, CachedRequestData> m_requestStack;
        QStack<CachedRequestData> m_cacheStack;
        int m_nextId; // returned by RequestMetaDataArtist & Track

        void init();
        void applyProxy();
        void checkBuffer() { if ( QHttp::bytesAvailable() ) m_buffer = QHttp::readAll(); }

        bool haveCachedCopy( QString url ) const;
        void putCachedCopy( QString cacheKey, const QByteArray& data );
};

#endif
