/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include "trackinfo.h"

#undef LOG
#undef LOGL
#define LOG(x, y)
#define LOGL(x, y)


TrackInfo::TrackInfo() :
    m_playCount( 0 ),
    m_duration( 0 ),
    m_source(Unknown)
{
}


TrackInfo::TrackInfo( const TrackInfo& that )
{
    populate( that );
}


TrackInfo::TrackInfo( const QDomElement& element )
{
    setArtist( element.namedItem( "artist" ).toElement().text() );
    setAlbum( element.namedItem( "album" ).toElement().text() );
    setTrack( element.namedItem( "track" ).toElement().text() );
    setDuration( element.namedItem( "duration" ).toElement().text() );
    setTimeStamp( element.namedItem( "timestamp" ).toElement().text() );
    setPlayCount( element.namedItem( "playcount" ).toElement().text().toInt() );
}


TrackInfo&
TrackInfo::operator=( const TrackInfo& that )
{
    // Check for self-assignment:
    if ( &that != this )
    {
        populate(that);
    }

    return *this;
}


TrackInfo::~TrackInfo()
{
}


void
TrackInfo::populate( const TrackInfo &trackinfo )
{
    setArtist( trackinfo.artist() );
    setAlbum( trackinfo.album() );
    setTrack( trackinfo.track() );
    setDuration( trackinfo.duration() );
    setPlayCount( trackinfo.playCount() );
    setPath( trackinfo.path() );
    setSource( trackinfo.source() );
    setMbId( trackinfo.mbId() );
    setTimeStamp( trackinfo.timeStamp() );
}


void
TrackInfo::clear()
{
    populate( TrackInfo() );
}


QDomElement
TrackInfo::toDomElement( QDomDocument& document ) const
{
    QDomElement item = document.createElement( "item" );

    QDomElement artist = document.createElement( "artist" );
    QDomText artistText = document.createTextNode( m_artist );
    artist.appendChild( artistText );
    item.appendChild( artist );

    QDomElement album = document.createElement( "album" );
    QDomText albumText = document.createTextNode( m_album );
    album.appendChild( albumText );
    item.appendChild( album );

    QDomElement title = document.createElement( "track" );
    QDomText titleText = document.createTextNode( m_track );
    title.appendChild( titleText );
    item.appendChild( title );

    QDomElement length = document.createElement( "duration" );
    QDomText lengthText = document.createTextNode( QString::number( m_duration ) );
    length.appendChild( lengthText );
    item.appendChild( length );

    QDomElement playtime = document.createElement( "timestamp" );
    QDomText playtimeText = document.createTextNode( m_timeStamp );
    playtime.appendChild( playtimeText );
    item.appendChild( playtime );

    QDomElement playcount = document.createElement( "playcount" );
    QDomText playcountText = document.createTextNode( QString::number( m_playCount ) );
    playcount.appendChild( playcountText );
    item.appendChild( playcount );

    return item;
}


bool
TrackInfo::sameAs( const TrackInfo &trackinfo )
{
    if ( artist() != trackinfo.artist() )
        return false;

    if ( track() != trackinfo.track() )
        return false;

    if ( album() != trackinfo.album() )
        return false;

    return true;
}

