/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef TRACKINFO_H
#define TRACKINFO_H

#include "common.h"
#include <QtCore>
#include <QtXml>

class MCEXP TrackInfo
{
    public:
        enum Source
        {
            Unknown = -1,
            Radio,
            Player,
            MediaDevice
        };

        TrackInfo();
        TrackInfo( const TrackInfo& that );
        TrackInfo( const QDomElement& element );
        TrackInfo& operator=( const TrackInfo& that );
        ~TrackInfo();

        void populate( const TrackInfo& trackinfo );
        void clear();
        bool sameAs( const TrackInfo &trackinfo );
        bool isEmpty() { return ( m_artist.isEmpty() && m_track.isEmpty() ); }

        QDomElement toDomElement( QDomDocument& document ) const;

        const QString artist() const { return m_artist; }
        void setArtist( QString artist ) { m_artist = artist; }

        const QString album() const { return m_album; }
        void setAlbum( QString album ) { m_album = album; }

        const QString track() const { return m_track; }
        void setTrack( QString track ) { m_track = track; }

        const int playCount() const { return m_playCount; }
        void setPlayCount( int playCount ) { m_playCount = playCount; }

        const int duration() const { return m_duration; }
        void setDuration( int duration ) { m_duration = duration; }
        void setDuration( QString duration ) { m_duration = duration.toInt(); }

        const QString mbId() const { return m_mbId; }
        void setMbId( QString mbId ) { m_mbId = mbId; }

        const QString path() const { return m_path; }
        void setPath( QString path ) { m_path = path; }

        const QString timeStamp() const { return m_timeStamp; }
        void setTimeStamp( QString timestamp ) { m_timeStamp = timestamp; }

        const Source source() const { return m_source; }
        void setSource( Source s ) { m_source = s; }

        QString toString() { return artist() + " - " + track(); }

    private:
        QString m_artist;
        QString m_album;
        QString m_track;

        int     m_playCount;
        int     m_duration;
        QString m_mbId;
        QString m_path;
        QString m_timeStamp;
        Source m_source;
};

#endif
