#include <QtGlobal>

#ifdef Q_WS_MAC

#include <QPainter>
#include <QStyleOption>
#include <QWidget>
#include <QDebug>

#include "macstyleoverrides.h"

static const QColor kLineGrey = QColor( 0x8c, 0x8c, 0x8c );

MacStyleOverrides::MacStyleOverrides() : QMacStyle()
{
	
}

QSize
MacStyleOverrides::sizeFromContents(
    ContentsType ct,
    const QStyleOption* opt,
    const QSize &csz,
    const QWidget* w) const
{
    if ( ct == CT_ToolButton )
    {
        // This doesn't really work to add space above the icons and it's
        // currently not used anywhere.
        QSize defaultSize = QMacStyle::sizeFromContents( ct, opt, csz, w );
        defaultSize.setHeight( defaultSize.height() + 5 );
        return defaultSize;
    }
    else
    {
        return QMacStyle::sizeFromContents( ct, opt, csz, w );
    }
}    

void 
MacStyleOverrides::drawControl(
    ControlElement pe,
    const QStyleOption* opt,
    QPainter* p,
    const QWidget* w) const
{
    if (pe == CE_Splitter)
    {
		p->setPen( kLineGrey );
		p->drawLine( opt->rect.topLeft(), opt->rect.bottomLeft() );
	}
    else
    {
		QMacStyle::drawControl(pe, opt, p, w);
    }
}

int
MacStyleOverrides::pixelMetric(
	PixelMetric metric,
	const QStyleOption* opt,
	const QWidget* widget) const
{
	switch (metric)
	{
		/* didn't help removing the 1px frame around the scrollarea
		case PM_DefaultFrameWidth:
			qDebug() << "FRAME 10!";
			return 10;
		*/
		
		default:
			return QMacStyle::pixelMetric( metric, opt, widget );
    }
}

void
MacStyleOverrides::drawPrimitive(
	PrimitiveElement pe,
	const QStyleOption* opt,
	QPainter* p,
	const QWidget* w) const
{
	// This removes the rounded border on the tab widget, draws a straight line
	// and fills the area with white.
	if ( pe == PE_FrameTabWidget )
	{
		p->setPen( QColor( 255, 255, 255, 255 ) );        
		p->setBrush( QColor( 255, 255, 255, 255 ) );
		p->drawRect( opt->rect );		

        p->setPen( kLineGrey );
		p->drawLine( opt->rect.topLeft(), opt->rect.topRight() );
    }
	else if ( pe == PE_FrameStatusBar )
	{
		// We don't want no borders
	}
	else
	{
		QMacStyle::drawPrimitive(pe, opt, p, w);
	}
}

QRect
MacStyleOverrides::subElementRect(
    SubElement sr,
    const QStyleOption *opt,
    const QWidget *widget ) const
{
    QRect rect;
    switch (sr)
    {
        case SE_TabWidgetTabContents:
        {
            rect = QMacStyle::subElementRect(sr, opt, widget);
            rect.adjust( 0, 0, 1, 2 );
        }
        break;
    
        default:
        {
            rect = QMacStyle::subElementRect(sr, opt, widget);
        }
        break;
    }
	
	return rect;
}

#endif // Q_WS_MAC
