/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include "itunesdevice.h"

#ifdef WIN32
    #include "windows.h"
    #include "shfolder.h"
#endif

const QString ITEM( "dict" ); //so we don't construct these QStrings all the time
const QString KEY( "key" ); //so we don't construct these QStrings all the time


QString
ITunesLibraryPath()
{
#ifdef Q_WS_MAC
    qDebug() << QFileInfo( QDir::homePath() + "/Music/iTunes/iTunes Music Library.xml" ).absoluteFilePath();
    return QFileInfo( QDir::homePath() + "/Music/iTunes/iTunes Music Library.xml" ).absoluteFilePath();
#endif

#ifdef WIN32
    QString path;

    // Use this for non-DOS-based Windowses
    char acPath[MAX_PATH];
    HRESULT h = SHGetFolderPathA( NULL, CSIDL_MYMUSIC,
                                  NULL, 0, acPath );
    if ( h == S_OK )
    {
        path = QString::fromLocal8Bit( acPath );
    }
    else
    {
        //LOG( 1, "Couldn't get Local Settings save path\n" );
    }

    return path + "/iTunes/iTunes Music Library.xml";
#endif

    return "/tmp/iTunes Music Library.xml";
}


ITunesDevice::ITunesDevice()
    : m_file( 0 )
{
    qDebug() << "Initialising iTunes MediaDevice";
    m_savePath = ITunesLibraryPath();
}


void
ITunesDevice::setupWatchers()
{
    m_fs = new QFileSystemWatcher( this );
    m_fs->addPath( ITunesLibraryPath() );

    #ifdef Q_WS_MAC
    m_fs->addPath( "/Volumes" );
    #endif

    connect( m_fs, SIGNAL( fileChanged( QString ) ),
             this,   SLOT( libraryChanged() ) );
    connect( m_fs, SIGNAL( directoryChanged( QString ) ),
             this,   SLOT( checkDevices() ) );
}


QList<TrackInfo>
ITunesDevice::tracks()
{
    qDebug() << "iTunesDevice::tracks start, " << QDateTime::currentDateTime().time().toString();

    QList<TrackInfo> tl;
    QFile file( m_savePath );
    if ( !file.open( QIODevice::ReadOnly | QIODevice::Text ) )
    {
        qDebug() << "Could not open iTunes Library" << m_savePath;
        return tl;
    }

    QTextStream stream( &file );
    stream.setCodec( "UTF-8" );

    QDomDocument d;
    if ( !d.setContent( stream.readAll() ) )
    {
        qDebug() << "Couldn't read file: " << m_savePath;
        return tl;
    }

    const QString ITEM( "dict" ); //so we don't construct these QStrings all the time
    const QString KEY( "key" ); //so we don't construct these QStrings all the time
    QDomNode plist = d.namedItem( "plist" ).firstChild();
    QDomNode n = plist.namedItem( "dict" ).firstChildElement( "dict" );

    for( ; !n.isNull(); n = n.nextSibling() )
    {
        TrackInfo t;
        if ( n.nodeName() != ITEM )
            continue;

        QDomNode k = n.firstChild();
        for( ; !k.isNull(); k = k.nextSibling() )
        {
            if ( k.nodeName() != KEY )
                continue;

            if ( k.toElement().text() == "Name" )
                t.setTrack( k.nextSibling().toElement().text().trimmed() );
            if ( k.toElement().text() == "Artist" )
                t.setArtist( k.nextSibling().toElement().text().trimmed() );
            if ( k.toElement().text() == "Album" )
                t.setAlbum( k.nextSibling().toElement().text().trimmed() );
            if ( k.toElement().text() == "Total Time" )
                t.setDuration( k.nextSibling().toElement().text().toInt() / 1000 );
            if ( k.toElement().text() == "Play Date UTC" )
            {
                QDateTime dt = QDateTime::fromString( k.nextSibling().toElement().text().trimmed(), Qt::ISODate );
//                 t.setTimeStamp( QString::number( dt.toTime_t() ) );
                t.setTimeStamp( dt.toString( "yyyy-MM-dd hh:mm:ss" ) );
            }
            if ( k.toElement().text() == "Play Count" )
                t.setPlayCount( k.nextSibling().toElement().text().toInt() );
        }

        if ( t.playCount() > 0 )
            tl << t;
    }

    file.close();

    qDebug() << "iTunesDevice::tracks end, " << QDateTime::currentDateTime().time().toString();

    return tl;
}


TrackInfo
ITunesDevice::nextTrack()
{
    TrackInfo t;

    if ( !m_node.isNull() )
        m_node = m_node.nextSibling();

    if ( !m_file )
    {
        m_file = new QFile( m_savePath );
        if ( !m_file->open( QIODevice::ReadOnly | QIODevice::Text ) )
        {
            qDebug() << "Could not open iTunes Library" << m_savePath;
            return t;
        }

        m_totalLines = 0;
        while ( !m_file->atEnd() )
        {
            m_totalLines++;
            m_file->readLine();
        }
        m_file->reset();

        QTextStream stream( m_file );
        stream.setCodec( "UTF-8" );

        m_dom = new QDomDocument();
        if ( !m_dom->setContent( m_file ) )
        {
            qDebug() << "Couldn't read file: " << m_savePath;
            return t;
        }

        QDomNode plist = m_dom->namedItem( "plist" ).firstChild();
        m_node = plist.namedItem( "dict" ).firstChildElement( "dict" );
    }

    while ( !m_node.isNull() )
    {
        while ( m_node.nodeName() != ITEM )
            m_node = m_node.nextSibling();

        QDomNode k = m_node.firstChild();
        for( ; !k.isNull(); k = k.nextSibling() )
        {
            if ( k.nodeName() != KEY )
                continue;

            if ( k.toElement().text() == "Name" )
                t.setTrack( k.nextSibling().toElement().text().trimmed() );
            if ( k.toElement().text() == "Artist" )
                t.setArtist( k.nextSibling().toElement().text().trimmed() );
            if ( k.toElement().text() == "Album" )
                t.setAlbum( k.nextSibling().toElement().text().trimmed() );
            if ( k.toElement().text() == "Total Time" )
                t.setDuration( k.nextSibling().toElement().text().toInt() / 1000 );
            if ( k.toElement().text() == "Play Date UTC" )
            {
                QDateTime dt = QDateTime::fromString( k.nextSibling().toElement().text().trimmed(), Qt::ISODate );
//                 t.setTimeStamp( QString::number( dt.toTime_t() ) );
                t.setTimeStamp( dt.toString( "yyyy-MM-dd hh:mm:ss" ) );
            }
            if ( k.toElement().text() == "Play Count" )
                t.setPlayCount( k.nextSibling().toElement().text().toInt() );
        }

        if ( t.playCount() > 0 )
        {
            qDebug() << "progress" << m_node.lineNumber();
            qDebug() << "total" << m_totalLines;
            emit progress( (float)( (float)m_node.lineNumber() / (float)m_totalLines ) * 100.0 );
            return t;
        }

        m_node = m_node.nextSibling();
    }

    m_file->close();
    delete m_file;
    m_file = 0;

    delete m_dom;

    return TrackInfo();
}


void
ITunesDevice::libraryChanged()
{
    while ( !QFileInfo( ITunesLibraryPath() ).exists() )
    #ifdef WIN32
        Sleep( 10 );
    #else
        sleep( 10 );
    #endif

    m_fs->addPath( ITunesLibraryPath() );

    emit deviceChanged();
}


// Checks the path given to see if the thing there is an iPod
void
ITunesDevice::checkDevices()
{
    qDebug() << "Rescanning devices";
    QStringList availUIDs;

    // For Mac, go through everything in /Volumes
    #ifdef Q_WS_MAC
    m_path = QDir( "/Volumes" );
    QStringList l = m_path.entryList( QDir::Dirs );

    for( int x = 0; x < l.count(); x++ )
    {
        while ( QFile::exists( "/" + m_path.dirName() + "/" + l.at( x ) + "/.autodiskmounted" ) )
        {
            sleep( 5 );
        }

        QFile file( "/" + m_path.dirName() + "/" + l.at( x ) + "/iPod_Control/Device/SysInfo" );
    #else
        QFile file( m_path.absolutePath() + "/iPod_Control/Device/SysInfo" );
    #endif

        if( file.open( QIODevice::ReadOnly | QIODevice::Text ) )
        {
            while ( file.bytesAvailable() )
            {
                QString s = QString( file.readLine() );
                if ( s.contains( "pszSerialNumber:" ) )
                {
                    QStringList tokens = s.split( " " );
                    availUIDs << tokens.at( 1 ).trimmed();
                }
            }
        }

    #ifdef Q_WS_MAC
    }
    #endif

    for ( int x = 0; x < availUIDs.count(); x++ )
    {
        if ( !m_availUIDs.contains( availUIDs.at( x ) ) )
            emit deviceAdded( "iPod_" + availUIDs.at( x ) );
    }

    m_availUIDs = availUIDs;
}


// Called when a new device is plugged in
void
ITunesDevice::forceDetection( const QString& path )
{
    m_availUIDs = QStringList();

    #ifdef WIN32
        m_path = QDir( path );
    #endif
    #ifdef Q_WS_MAC
        m_path = QDir( "/Volumes" );
    #endif

    return checkDevices();
}


Q_EXPORT_PLUGIN2( mediadevice, ITunesDevice )
