/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include <QtGui>

#include "mp3transcode.h"
#include "containerutils.h"
#include "Loqqer.h"

#define PLAYBACKBUFFER 2048
struct mpstr_tag mpeg;

MP3Transcode::MP3Transcode()
{
    gLogger.Init( savePath( "transcode.log" ), false );
    gLogger.SetLevel( 4 );

    LOGL( 3, "Initialising MP3 Transcoding" );

    if ( !InitMP3( &mpeg ) )
    {
        // Nominated for best-comment-of-the-decade-award.
        LOGL( 1, "Something went wrong when initiliasing mpglib. "
            " God knows what. Chris can read the \"best documentation "
            " there is\" to find out. :P" );
    }
}


void
MP3Transcode::dataAvailable( const QByteArray &buffer )
{
    m_mutex.lock();
    m_buffer.append( buffer );

    if ( m_buffer.size() > PLAYBACKBUFFER )
    {
        QByteArray outputBuffer;
        char tempBuffer[16384];
        int size;

        int result = decodeMP3( &mpeg, (unsigned char*)m_buffer.data(), PLAYBACKBUFFER, tempBuffer, sizeof( tempBuffer ), &size );
        m_buffer.remove( 0, PLAYBACKBUFFER );

        while ( result == MP3_OK )
        {
            for ( int i = 0; i < ( size / 2 ); i++ )
            {
                outputBuffer.append( tempBuffer[i * 2] );
                outputBuffer.append( tempBuffer[i * 2 + 1] );
            }

            result = decodeMP3( &mpeg, NULL, 0, tempBuffer, sizeof( tempBuffer ), &size );
        }

        emit newData( outputBuffer );
    }

    m_mutex.unlock();
}


Q_EXPORT_PLUGIN2( transcode, MP3Transcode )
