/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include <QtGui>
#include <QTimer>

#include <containerutils.h>
#include "recommenddialog.h"
#include "settingsserviceinterface.h"
#include "webserviceinterface.h"


RecommendDialog::RecommendDialog( QWidget *parent )
    : QDialog( parent, Qt::Dialog )
{
    m_webService = webService();
    Q_ASSERT( m_webService != NULL );

    ui.setupUi( this );

    ui.userEdit->setEditText( tr( "Loading..." ) );
    ui.userEdit->setEnabled( false );
    ui.recommendTypeBox->setCurrentIndex( settingsService()->currentUser()->lastRecommendType( 1 ) ); // set track as default

    connect( ui.recommendTypeBox, SIGNAL( activated( int ) ), this, SLOT( recommendTypeChanged( int ) ) );
    connect( ui.okButton, SIGNAL( clicked() ), this, SLOT( okPressed() ) );
    connect( m_webService, SIGNAL( friendsResult( QString, QStringList ) ), this, SLOT( updateFriends( QString, QStringList ) ) );

    m_webService->friends();
}


void
RecommendDialog::setSong( MetaData metaData )
{
    m_metaData = metaData;

    ui.recommendTypeBox->setItemText( 0, QString( tr( "Artist: %1" ) ).arg( metaData.artist() ) );
    ui.recommendTypeBox->setItemText( 1, QString( tr( "Track: %1 - %2" ) ).arg( metaData.artist() ).arg( metaData.track() ) );
    ui.recommendTypeBox->setItemText( 2, QString( tr( "Album: %1 - %2" ) ).arg( metaData.artist() ).arg( metaData.album() ) );
}


void
RecommendDialog::updateFriends( const QString& username, const QStringList& friends )
{
    if ( username.toLower() != m_webService->currentUsername().toLower() )
        return;

    ui.userEdit->addItems( friends );
    ui.userEdit->setEditText( "" );
    ui.userEdit->setEnabled( true );
    ui.userEdit->setFocus();
}


void
RecommendDialog::okPressed()
{
    switch( ui.recommendTypeBox->currentIndex() )
    {
        case 0:
            m_webService->recommend( ItemArtist, ui.userEdit->currentText(), ui.messageEdit->toPlainText(), m_metaData.artist() );
            break;

        case 1:
            m_webService->recommend( ItemTrack, ui.userEdit->currentText(), ui.messageEdit->toPlainText(),
                                     m_metaData.artist(), m_metaData.track() );
            break;

        case 2:
            m_webService->recommend( ItemAlbum, ui.userEdit->currentText(), ui.messageEdit->toPlainText(),
                                     m_metaData.artist(), m_metaData.album() );
            break;

    }
}


void
RecommendDialog::recommendTypeChanged( int type )
{
    settingsService()->currentUser()->setLastRecommendType( ui.recommendTypeBox->currentIndex() );
}
