/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef SEARCH_EXTENSION_H
#define SEARCH_EXTENSION_H

#include <QObject>
#include <QFrame>
#include <QBuffer>

#include <extensioninterface.h>
#include <inputinterface.h>
#include <settingsserviceinterface.h>
#include <webserviceinterface.h>

#include "watermarkwidget.h"

#ifdef Q_WS_MAC
    #include "ui_search_mac.h"
#else
    #include "ui_search_win.h"
#endif

class SearchExtension : public ExtensionInterface
{
    Q_OBJECT
    Q_INTERFACES( ExtensionInterface )

    public:
        SearchExtension();

        QString name() const;
        QString version() const;
        QString tabCaption() const { return tr( "Tune in" ); }

        virtual bool hasGui() { return true; }

        QWidget* gui();
        bool guiEnabled() { return ( m_widget != 0 ); }

        QWidget* owner() { return m_parent; }
        void setOwner( QWidget* parent ) { m_parent = parent; setParent( parent ); }

        virtual bool eventFilter( QObject* watched, QEvent* event );
        
        InputInterface *m_iInput;
        SettingsServiceInterface *m_iSettings;
        WebServiceInterface *m_iWebService;

    signals:
        void stationFocused( const QString& url );

    private:
        void initGui();
        void restoreSearchField();
        
        Ui::SearchExtension ui;
        QWidget* m_widget;
        QWidget* m_parent;
        QScrollArea* m_scrollArea;
        QMovie m_spinnerMovie;
        
        int m_type;
        int m_currentCloudType;
        
        QString m_lastSearch;

        bool m_topTags;
        bool m_clearSearchOnClick;
        bool m_searching;

    private slots:
        void search();

        void play();
        QString stationUrl();

        //void selectedItemClicked( int index );

        void searchFieldChanged();
        void topTags();
        void searchResults( const QList<QPair<QString, int> >& items, const QString& token = QString() );
        void itemClicked( int index );
        void clearSelection();
};

#endif
