/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include <QtGui>

#ifdef WIN32
    #include <windows.h>
#endif

#include "containerutils.h"
#include "settingsdialog.h"
#include "lastfmapplication.h"

#include "Loqqer.h"

// This is a terrible way of entering strings but I couldn't get VS to recognise
// the Unicode chars in any other way.
static const unsigned char kChinese[] =
    { 0xE4, 0xB8, 0xAD, 0xE6, 0x96, 0x87, 0x0 };
static const unsigned char kKorean[] =
    { 0xED, 0x95, 0x9C, 0xEA, 0xB5, 0xAD, 0xEC, 0x96, 0xB4, 0x0 }; 
static const unsigned char kRussian[] =
    { 0xD0, 0xA0, 0xD1, 0x83, 0xD1, 0x81, 0xD1, 0x81, 0xD0, 0xBA,
      0xD0, 0xB8, 0xD0, 0xB9, 0x0 };
static const unsigned char kJapanese[] =
    { 0xE6, 0x97, 0xA5, 0xE6, 0x9C, 0xAC, 0xE8, 0xAA, 0x9E, 0x0 }; 

SettingsDialog::SettingsDialog( QWidget *parent )
    : QDialog( parent )
{
    m_iSettings = settingsService();

    ui.setupUi( this );

	#ifdef Q_WS_MAC
		setWindowTitle( tr( "Last.fm Preferences" ) );
	#else
		setWindowTitle( tr( "Last.fm Options" ) );
	#endif
    
	// Create pages and add them to stack widget
    QWidget* accountWidget = new QWidget();
    ui_account.setupUi( accountWidget );
    ui.pageStack->addWidget( accountWidget );

    QWidget* radioWidget = new QWidget();
    ui_radio.setupUi( radioWidget );
    ui.pageStack->addWidget( radioWidget );

    QWidget* scrobWidget = new QWidget();
    ui_scrobbling.setupUi( scrobWidget );
    ui.pageStack->addWidget( scrobWidget );

    QWidget* connWidget = new QWidget();
    ui_connection.setupUi( connWidget );
    ui.pageStack->addWidget( connWidget );

    #ifndef WIN32
        ui_connection.crashReportCheck->hide();
    #endif

    // Add icons to user icon dropdown
    QPixmap pixmap( dataPath( "icons/user_red.png" ) );
    ui_account.colourCombo->setItemIcon( 0, pixmap );
    pixmap.load( dataPath( "icons/user_blue.png" ) );
    ui_account.colourCombo->setItemIcon( 1, pixmap );
    pixmap.load( dataPath( "icons/user_green.png" ) );
    ui_account.colourCombo->setItemIcon( 2, pixmap );
    pixmap.load( dataPath( "icons/user_orange.png" ) );
    ui_account.colourCombo->setItemIcon( 3, pixmap );
    pixmap.load( dataPath( "icons/user_black.png" ) );
    ui_account.colourCombo->setItemIcon( 4, pixmap );

    // Add languages to language drop-down
    ui_account.languageCombo->addItem(
        "English", qtLanguageToLangCode( QLocale::English ) );
    ui_account.languageCombo->addItem(
        "Franais", qtLanguageToLangCode( QLocale::French ) );
    ui_account.languageCombo->addItem(
        "Italiano", qtLanguageToLangCode( QLocale::Italian ) );
    ui_account.languageCombo->addItem(
        "Deutsch", qtLanguageToLangCode( QLocale::German ) );
    ui_account.languageCombo->addItem(
        "Espaol", qtLanguageToLangCode( QLocale::Spanish ) );
    ui_account.languageCombo->addItem(
        "Portugus", qtLanguageToLangCode( QLocale::Portuguese ) );
    ui_account.languageCombo->addItem(
        "Polski", qtLanguageToLangCode( QLocale::Polish ) );
    ui_account.languageCombo->addItem(
        QString::fromUtf8( reinterpret_cast<const char*>( kRussian ) ),
        qtLanguageToLangCode( QLocale::Russian ) );
    ui_account.languageCombo->addItem(
        QString::fromUtf8( reinterpret_cast<const char*>( kChinese ) ),
        qtLanguageToLangCode( QLocale::Chinese ) );
    ui_account.languageCombo->addItem(
        QString::fromUtf8( reinterpret_cast<const char*>( kKorean ) ),
        qtLanguageToLangCode( QLocale::Korean ) );

    // Add icons to sidebar
    pixmap.load( dataPath( "/icons/options_account.png" ) );
    //pixmap.scaled( 48, 48, Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
    ui.pageList->item( 0 )->setIcon( pixmap );

    pixmap.load( dataPath( "/icons/options_radio.png" ) );
    //pixmap.scaled( 48, 48, Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
    ui.pageList->item( 1 )->setIcon( pixmap );

    #ifdef HIDE_RADIO
        ui.pageList->setItemHidden( ui.pageList->item( 1 ), true );
    #endif // HIDE_RADIO    

    pixmap.load( dataPath( "/icons/options_scrobbling.png" ) );
    //pixmap.scaled( 48, 48, Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
    ui.pageList->item( 2 )->setIcon( pixmap );

    pixmap.load( dataPath( "/icons/options_connection.png" ) );
    //pixmap.scaled( 48, 48, Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
    ui.pageList->item( 3 )->setIcon( pixmap );

    ui_account.usernameEdit->setEnabled( false );
    
    #ifdef HIDE_RADIO
        // Need to disable this as the language choice is hardcoded at startup
        // (see LastfmApp::initTranslator) for the jp version.
        ui_account.languageCombo->hide();
        ui_account.languageLabel->hide();
    #endif

    connect( ui.pageList, SIGNAL( currentRowChanged( int ) ), this, SLOT( pageChanged( int ) ) );
    connect( ui.okButton, SIGNAL( clicked() ), this, SLOT( okPressed() ) );
    connect( ui.applyButton, SIGNAL( clicked() ), this, SLOT( applyPressed() ) );
    connect( ui_connection.clearCacheButton, SIGNAL( clicked() ), this, SLOT( clearCache() ) );

    connect( ui_account.usernameEdit, SIGNAL( textChanged( QString ) ), this, SLOT( configChanged() ) );
    connect( ui_account.passwordEdit, SIGNAL( textChanged( QString ) ), this, SLOT( configChanged() ) );
    connect( ui_account.rememberPassCheck, SIGNAL( toggled( bool ) ), this, SLOT( configChanged() ) );
    connect( ui_account.colourCombo, SIGNAL( currentIndexChanged( int ) ), this, SLOT( configChanged() ) );
    connect( ui_account.languageCombo, SIGNAL( currentIndexChanged( int ) ), this, SLOT( configChanged() ) );
    connect( ui_radio.resumeCheckBox, SIGNAL( toggled( bool ) ), this, SLOT( configChanged() ) );
    connect( ui_radio.cardBox, SIGNAL( currentIndexChanged( int ) ), this, SLOT( configChanged() ) );
    connect( ui_radio.systemBox, SIGNAL( currentIndexChanged( int ) ), this, SLOT( configChanged() ) );
    connect( ui_scrobbling.scrobblingEnabledCheck, SIGNAL( toggled( bool ) ), this, SLOT( configChanged() ) );
    connect( ui_scrobbling.dirTree, SIGNAL( dataChanged() ), this, SLOT( configChanged() ) );
    connect( ui_connection.proxyBox, SIGNAL( toggled( bool ) ), this, SLOT( configChanged() ) );
    connect( ui_connection.proxyHostEdit, SIGNAL( textChanged( QString ) ), this, SLOT( configChanged() ) );
    connect( ui_connection.proxyPortEdit, SIGNAL( textChanged( QString ) ), this, SLOT( configChanged() ) );
    connect( ui_connection.proxyUsernameEdit, SIGNAL( textChanged( QString ) ), this, SLOT( configChanged() ) );
    connect( ui_connection.proxyPasswordEdit, SIGNAL( textChanged( QString ) ), this, SLOT( configChanged() ) );
    connect( ui_connection.downloadMetadataCheck, SIGNAL( toggled( bool ) ), this, SLOT( configChanged() ) );
    connect( ui_connection.crashReportCheck, SIGNAL( toggled( bool ) ), this, SLOT( configChanged() ) );
}

int
SettingsDialog::exec()
{
    if ( !m_iSettings->currentUsername().isEmpty() )
    {
        originalUsername = m_iSettings->currentUser()->username();
        originalPassword = m_iSettings->currentUser()->password();
        originalProxyHost = m_iSettings->proxyHost();
        originalProxyUsername = m_iSettings->proxyUsername();
        originalProxyPassword = m_iSettings->proxyPassword();
        originalProxyPort = m_iSettings->proxyPort();
        originalProxyUsage = m_iSettings->useProxy();
        originalSoundCard = m_iSettings->soundCard();
        originalSoundSystem = m_iSettings->soundSystem();
    }

    pageChanged( 0 );
    ui.applyButton->setEnabled( false );
    return QDialog::exec();
} 

void
SettingsDialog::populateAccount()
{
    UserSettingsServiceInterface* user = m_iSettings->currentUser();

    ui_account.usernameEdit->setText( user->username() );
    if ( !user->password().isEmpty() )
        ui_account.passwordEdit->setText( "********" );
    ui_account.rememberPassCheck->setChecked( user->rememberPass() );

    ui_account.colourCombo->setCurrentIndex( user->icon() < ui_account.colourCombo->count() ? user->icon() : 0 );

    #ifndef HIDE_RADIO
        QString langCode = m_iSettings->appLanguage();
        for( int i = 0; i < ui_account.languageCombo->count(); ++i )
        {
            QString code = ui_account.languageCombo->itemData( i ).toString();
            if ( code == langCode )
            {
                ui_account.languageCombo->setCurrentIndex( i );
            }
        }
    #endif // HIDE_RADIO
}


void
SettingsDialog::populateRadio()
{
    ui_radio.resumeCheckBox->setChecked( m_iSettings->currentUser()->resumePlayback() );

    ui_radio.systemBox->clear();
    ui_radio.cardBox->clear();

    ui_radio.systemBox->addItems( playbackService()->soundSystems() );
    ui_radio.cardBox->addItems( playbackService()->devices() );

    ui_radio.systemBox->setCurrentIndex( m_iSettings->soundSystem() >= 0 ? m_iSettings->soundSystem() : 0 );
    ui_radio.cardBox->setCurrentIndex( m_iSettings->soundCard() >= 0 ? m_iSettings->soundCard() : 0 );
}


void
SettingsDialog::populateScrobbling()
{
    UserSettingsServiceInterface* user = m_iSettings->currentUser();

    ui_scrobbling.scrobblingEnabledCheck->setChecked( user->logToProfile() );
    ui_scrobbling.dirTree->setExclusions( user->excludedDirs() );
}


void
SettingsDialog::populateConnection()
{
    ui_connection.proxyBox->setChecked( m_iSettings->useProxy() );
    ui_connection.proxyHostEdit->setText( m_iSettings->proxyHost() );
    ui_connection.proxyPortEdit->setText( QString().setNum( m_iSettings->proxyPort() ) );
    ui_connection.proxyUsernameEdit->setText( m_iSettings->proxyUsername() );
    ui_connection.proxyPasswordEdit->setText( m_iSettings->proxyPassword() );
    ui_connection.downloadMetadataCheck->setChecked( m_iSettings->currentUser()->metadataEnabled() );
    ui_connection.crashReportCheck->setChecked( m_iSettings->currentUser()->crashReportingEnabled() );
}


void
SettingsDialog::pageChanged( int currentRow )
{
    switch ( currentRow )
    {
        case 0:
            populateAccount();
            break;

        case 1:
            populateRadio();
            break;

        case 2:
            populateScrobbling();
            break;

        case 3:
            populateConnection();
            break;
            
        default:
            // It's an extension
            int idx = currentRow - 4;
            extensions[idx]->populateSettings();
            break;

    }

    ui.pageList->setCurrentRow( currentRow );
    ui.pageStack->setCurrentIndex( currentRow + 1 );
    m_pagesToSave.insert( currentRow );
}


void
SettingsDialog::applyPressed()
{
    // Do hardcoded sections
    if ( m_pagesToSave.contains( 0 ) ) { saveAccount(); }
    if ( m_pagesToSave.contains( 1 ) ) { saveRadio(); }
    if ( m_pagesToSave.contains( 2 ) ) { saveScrobbling(); }
    if ( m_pagesToSave.contains( 3 ) ) { saveConnection(); }

    for ( int i = 0; i < 4; ++i )
    {
        m_pagesToSave.remove( i );
    }
    
    // Do extensions
    foreach( int page, m_pagesToSave )
    {
        int idx = page - 4;
        extensions[idx]->saveSettings();
    }
    
    //m_iSettings->save();

    ui.applyButton->setEnabled( false );
}


void
SettingsDialog::saveAccount()
{
    m_iSettings->setCurrentUsername( ui_account.usernameEdit->text() );

    UserSettingsServiceInterface* user = m_iSettings->currentUser();
    int iconIndex = ui_account.colourCombo->currentIndex() >= 0 ? ui_account.colourCombo->currentIndex() : 0;

    user->setPassword( ui_account.passwordEdit->text() );
    user->setIcon( static_cast<UserIconColour>( iconIndex ) );
    user->setRememberPass( ui_account.rememberPassCheck->isChecked() );

    #ifndef HIDE_RADIO
        // Store language in config
        int langIdx = ui_account.languageCombo->currentIndex();
        QString langCode = ui_account.languageCombo->itemData( langIdx ).toString();
        QString oldLang = m_iSettings->appLanguage();    
        
        if ( langCode != oldLang )
        {
            QMessageBox dlg( QMessageBox::Information,
                tr( "Restart needed" ),
                tr( "You need to restart the application for the language "
                    "change to take effect." ),
                QMessageBox::Ok, this );
            dlg.exec();
        }

        m_iSettings->setAppLanguage( langCode );
    #endif
}


void
SettingsDialog::saveRadio()
{
    UserSettingsServiceInterface* user = m_iSettings->currentUser();

    user->setResumePlayback( ui_radio.resumeCheckBox->isChecked() );

    m_iSettings->setSoundCard( ui_radio.cardBox->currentIndex() >= 0 ? ui_radio.cardBox->currentIndex() : 0 );
    m_iSettings->setSoundSystem( ui_radio.systemBox->currentIndex() >= 0 ? ui_radio.systemBox->currentIndex() : 0 );
}


void
SettingsDialog::saveScrobbling()
{
    UserSettingsServiceInterface* user = m_iSettings->currentUser();

    user->setLogToProfile( ui_scrobbling.scrobblingEnabledCheck->isChecked() );

    QStringList exclusions = ui_scrobbling.dirTree->getExclusions();
    user->setExcludedDirs(exclusions);
}


void
SettingsDialog::saveConnection()
{
    m_iSettings->setProxyUsername( ui_connection.proxyUsernameEdit->text() );
    m_iSettings->setProxyPassword( ui_connection.proxyPasswordEdit->text() );
    m_iSettings->setProxyHost( ui_connection.proxyHostEdit->text() );
    m_iSettings->setProxyPort( ui_connection.proxyPortEdit->text().toInt() );
    m_iSettings->setUseProxy( ui_connection.proxyBox->isChecked() );
    m_iSettings->currentUser()->setMetadataEnabled( ui_connection.downloadMetadataCheck->isChecked() );
    m_iSettings->currentUser()->setCrashReportingEnabled( ui_connection.crashReportCheck->isChecked() );
}


void
SettingsDialog::okPressed()
{
    QString password = ui_account.passwordEdit->text();

    if ( password.isEmpty() || password == "********" )
        password = originalPassword;
    else
        password = MD5Digest( ui_account.passwordEdit->text().toAscii() );

    bool reconnect = false;
    if ( m_pagesToSave.contains( 0 ) )
    {
        reconnect = ui_account.usernameEdit->text() != originalUsername ||
                    password != originalPassword;
    }
    if ( !reconnect && m_pagesToSave.contains( 3 ) )
    {
        reconnect =  ui_connection.proxyHostEdit->text() != originalProxyHost ||
                     ui_connection.proxyUsernameEdit->text() != originalProxyUsername ||
                     ui_connection.proxyPasswordEdit->text() != originalProxyPassword ||
                     ui_connection.proxyPortEdit->text().toInt() != originalProxyPort ||
                     ui_connection.proxyBox->isChecked() != originalProxyUsage;
    }
    
    bool reaudio = false;
    if ( m_pagesToSave.contains( 1 ) )
    {
        reaudio = ui_radio.cardBox->currentIndex() != originalSoundCard ||
                   ui_radio.systemBox->currentIndex() != originalSoundSystem;
    }

    applyPressed();

    // EJ: why are these stored just before exiting?
    originalUsername = m_iSettings->currentUser()->username();
    originalPassword = m_iSettings->currentUser()->password();
    originalProxyHost = m_iSettings->proxyHost();
    originalProxyUsername = m_iSettings->proxyUsername();
    originalProxyPassword = m_iSettings->proxyPassword();
    originalProxyPort = m_iSettings->proxyPort();
    originalProxyUsage = m_iSettings->useProxy();

    m_iSettings->save( reconnect, reaudio );
}


void
SettingsDialog::configChanged()
{
    ui.applyButton->setEnabled( true );
}

void
SettingsDialog::addExtension( ExtensionInterface* ext )
{
    extensions.append( ext );
    
    ext->setOwner( this );

    ui.pageStack->addWidget( ext->settingsPane() );
    ui.pageList->addItem( ext->tabCaption() );

    int idx = ui.pageList->count() - 1;
    ui.pageList->item( idx )->setIcon( *ext->settingsIcon() );

    // God knows why I have to give it the name again, but I do
    ui.pageList->item( idx )->setText( ext->tabCaption() );

    connect( ext,  SIGNAL( settingsChanged() ),
             this, SLOT( configChanged() ) );
}

void
SettingsDialog::clearCache()
{
    // Doesn't really belong here but...

    int sure = QMessageBox::question( this,
        tr( "Confirm" ),
        tr( "Are you sure you want to delete all\ncached images and bios?" ),
        QMessageBox::Yes,
        QMessageBox::No );

    if ( sure == QMessageBox::Yes )
    {
        QApplication::setOverrideCursor( QCursor( Qt::WaitCursor ) );

        QDir dir( savePath( "cache/" ) );

        QStringList files = dir.entryList( QDir::Files );
        bool success = true;
        int count = 0;
        foreach( QString filename, files )
        {
            bool deleted = dir.remove( filename );
            if ( deleted ) { count++; }
            success &= deleted;
            
            if ( count % 25 == 0 )
            {
                QApplication::processEvents();
            }
        }

        QApplication::restoreOverrideCursor();

        QString result;
        if ( success )
        {
            result = tr( "Cache emptied. %1 files deleted." ).arg( count );
        }
        else
        {
            result = tr( "Not all items could be deleted. You might need "
                "to delete some items manually from\n'%1'." )
                .arg( dir.absolutePath() );
        }

        QMessageBox::information( this,
            tr( "Finished" ),
            result,
            QMessageBox::Ok );

        LOGL( 3, "Emptied cache, files: " << count << ", success: " << success );
    }
}
