/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef SETTINGSSERVICE_H
#define SETTINGSSERVICE_H

#define VERSION "1.1.3.0"

#include <settingsserviceinterface.h>

#include <QtCore>

class UserSettingsService : public UserSettingsServiceInterface
{
    Q_OBJECT

    public:
        UserSettingsService( QString username = QString() );

        void loadUsername( QString username );
        QString username() const { return m_username; }

        QString password() const;
        void setPassword( QString password );

        UserIconColour icon() const;
        void setIcon( UserIconColour colour );

        bool logToProfile() const;
        void setLogToProfile( bool state );

        bool sidebarEnabled() const;
        void setSidebarEnabled( bool state );

        int lastTagType( int type = 0 ) const { return m_config->value( "lasttagtype", type ).toInt(); }
        void setLastTagType( int type ) { m_config->setValue( "lasttagtype", type ); }
        int lastRecommendType( int type = 0 ) const { return m_config->value( "lastrecommendtype", type ).toInt(); }
        void setLastRecommendType( int type ) { m_config->setValue( "lastrecommendtype", type ); }

        bool rememberPass() const;
        void setRememberPass( bool remember );

        bool resumePlayback() const { return ( m_config->value( "resumeplayback" ).toInt() == 1 ); }
        void setResumePlayback( bool enabled ) { m_config->setValue( "resumeplayback", enabled ? "1" : "0" ); emit userChanged( username() ); }

        QString resumeStation() const { return m_config->value( "resumestation" ).toString(); }
        void setResumeStation( QString station ) { m_config->setValue( "resumestation", station ); emit userChanged( username() ); }

        void addRecentStation( QString url, QString name );
        void clearRecentStations( bool emitting );
        QStringList recentStations() const;

        QString stationName( QString url ) const;
        void setStationName( QString url, QString name );

        void setExcludedDirs( QStringList dirs );
        QStringList excludedDirs() const;
        void setIncludedDirs( QStringList dirs );
        QStringList includedDirs() const;

        void setMetadataEnabled( bool enabled );
        bool metadataEnabled();

        void setCrashReportingEnabled( bool enabled );
        bool crashReportingEnabled();

        QSettings* qSettings() { return m_config; }

    signals:
        void userChanged(QString username);
        void historyChanged();

    public slots:
        void save();

    private:
        QSettings *m_config;
        QString m_username;
};


class SettingsService : public SettingsServiceInterface
{
    Q_OBJECT
    Q_INTERFACES( SettingsServiceInterface )

    public:
        SettingsService();
        ~SettingsService();

        QString version() { return VERSION; }
        QByteArray appState() { return m_config->value( "appState" ).toByteArray(); }
        void setAppState( QByteArray state ) { m_config->setValue( "appState", state ); }
        QByteArray splitterState() { return m_config->value( "splitterState" ).toByteArray(); }
        void setSplitterState( QByteArray state ) { m_config->setValue( "splitterState", state ); }

        QPoint position() { return m_config->value( "position" ).toPoint(); }
        void setPosition( QPoint position ) { m_config->setValue( "position", position ); }
        QSize size() { return m_config->value( "size" ).toSize(); }
        void setSize( QSize size ) { m_config->setValue( "size", size ); }

        QStringList allUsers() { return m_configUsers->childGroups(); }
        QString currentUsername() { return m_configUsers->value( "CurrentUser", "" ).toString(); }
        void setCurrentUsername( QString username ) {
            m_configUsers->setValue( "CurrentUser", username );
            emit currentUserChanged(currentUser());
            emit userSwitch(currentUser());
        }
        UserSettingsServiceInterface* currentUser() {
            Q_ASSERT(currentUsername() != ""); return m_userList.value(currentUsername()); }
        UserSettingsServiceInterface* user( QString username );
        bool deleteUser( QString username );
        bool existingUser( QString username ) { return m_configUsers->contains( QString( "%1/Password" ).arg( username ) ); }

        QStringList allPlugins();

        // Returns "" if value not found
        QString pluginVersion( QString pluginId );
        QString pluginPlayerPath( QString pluginId );
        void setPluginPlayerPath( QString pluginId, QString path );

        QStringList allMediaDevices();

        // Returns "" if value not found
        QString MediaDeviceUser( const QString& uid );
        void addMediaDevice( const QString& uid, const QString& username );

        int volume() { return m_config->value( "volume", 50 ).toInt(); }
        void setVolume( int volume ) { m_config->setValue( "volume", volume ); }
        int soundCard() { return m_config->value( "soundcard", 0 ).toInt(); }
        void setSoundCard( int card ) { m_config->setValue( "soundcard", card ); }

        int soundSystem() { return m_config->value( "soundsystem", 0 ).toInt(); }
        void setSoundSystem( int system ) { m_config->setValue( "soundsystem", system ); }
        int externalSoundSystem();

        QString proxyUsername() { return m_config->value( "ProxyUser" ).toString(); }
        void setProxyUsername( QString username ) { m_config->setValue( "ProxyUser", username ); }
        QString proxyPassword() { return m_config->value( "ProxyPassword" ).toString(); }
        void setProxyPassword( QString password ) { m_config->setValue( "ProxyPassword", password ); }

        QString proxyHost() { return m_config->value( "ProxyHost" ).toString(); }
        void setProxyHost( QString host ) { m_config->setValue( "ProxyHost", host ); }
        int proxyPort() { return m_config->value( "ProxyPort" ).toInt(); }
        void setProxyPort( int port ) { m_config->setValue( "ProxyPort", port ); }

        bool useProxy() { return ( m_config->value( "ProxyEnabled" ).toInt() == 1 ); }
        void setUseProxy( bool enabled ) { m_config->setValue( "ProxyEnabled", enabled ? "1" : "0" ); }

        QString browser();
        void setBrowser( QString browser );

        // This should be a string for legacy reasons
        bool firstRun() { return m_configLM->value( "FirstRun", "1" ).toBool(); }
        void firstRunDone() { m_configLM->setValue( "FirstRun", "0" ); }
        bool bootstrap() { return m_configLM->value( "BootStrapDone", "0" ).toBool(); }
        void bootstrapDone() { m_configLM->setValue( "BootStrapDone", "1" ); }

        void setDontAsk( const QString op, bool value );
        bool dontAsk( const QString op );

        // Use one of our pre-defined 2-letter language codes
        void setAppLanguage( QString langCode ); 
        QString appLanguage(); 

        UserIconColour getFreeColour();

    public slots:
        void save( bool restartConnection = false, bool restartAudio = false );

    signals:
        void reconnect();
        void resetAudio();

        // Some property belonging to the current user changed
        void currentUserChanged( UserSettingsServiceInterface* user );

        // The current user changed to a different user
        void userSwitch( UserSettingsServiceInterface* newUser );

    private:
        QSettings* m_config;
        QSettings* m_configUsers;
        QSettings* m_configPlugins;
        QSettings* m_configMediaDevices;
        QSettings* m_configLM;

        QHash<QString, UserSettingsService*> m_userList;

        void loadUsers();

    private slots:
        void userChanged(QString username);
};

#endif
