/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef SETTINGSSERVICE_INTERFACE_H
#define SETTINGSSERVICE_INTERFACE_H

#include <QObject>
#include <QPoint>
#include <QSize>
#include <QSettings>

enum UserIconColour
{
    eNone = -1,
    eRed = 0,
    eBlue,
    eGreen,
    eOrange,
    eBlack
};

class UserSettingsServiceInterface: public QObject
{
    public:
        virtual ~UserSettingsServiceInterface() {}

        void loadUsername( QString username );
        virtual QString username() const = 0;

        virtual QString password() const = 0;
        virtual void setPassword( QString password ) = 0;

        virtual UserIconColour icon() const = 0;
        virtual void setIcon( UserIconColour colour ) = 0;

        virtual bool logToProfile() const = 0;
        virtual void setLogToProfile( bool state ) = 0;

        virtual bool sidebarEnabled() const = 0;
        virtual void setSidebarEnabled( bool state ) = 0;

        virtual int lastTagType( int type = 0 ) const = 0;
        virtual void setLastTagType( int type ) = 0;
        virtual int lastRecommendType( int type = 0 ) const = 0;
        virtual void setLastRecommendType( int type ) = 0;

        virtual bool rememberPass() const = 0;
        virtual void setRememberPass( bool remember ) = 0;

        virtual bool resumePlayback() const = 0;
        virtual void setResumePlayback( bool enabled ) = 0;
        virtual QString resumeStation() const = 0;
        virtual void setResumeStation( QString station ) = 0;

        virtual void addRecentStation( QString url, QString name ) = 0;
        virtual void clearRecentStations( bool emitting ) = 0;
        virtual QStringList recentStations() const = 0;

        virtual QString stationName( QString url ) const = 0;
        virtual void setStationName( QString url, QString name ) = 0;

        virtual void setExcludedDirs( QStringList dirs ) = 0;
        virtual QStringList excludedDirs() const = 0;
        virtual void setIncludedDirs( QStringList dirs ) = 0;
        virtual QStringList includedDirs() const = 0;

        virtual void setMetadataEnabled( bool enabled ) = 0;
        virtual bool metadataEnabled() = 0;

        virtual void setCrashReportingEnabled( bool enabled ) = 0;
        virtual bool crashReportingEnabled() = 0;

        virtual QSettings* qSettings() = 0;

    public slots:
        virtual void save() = 0;

};

class SettingsServiceInterface : public QObject
{
    public:
        virtual ~SettingsServiceInterface() {}

        virtual QString version() = 0;
        virtual QByteArray appState() = 0;
        virtual void setAppState( QByteArray state ) = 0;
        virtual QByteArray splitterState() = 0;
        virtual void setSplitterState( QByteArray state ) = 0;

        virtual QPoint position() = 0;
        virtual void setPosition( QPoint position ) = 0;
        virtual QSize size() = 0;
        virtual void setSize( QSize size ) = 0;

        virtual QStringList allUsers() = 0;
        virtual QString currentUsername() = 0;
        virtual void setCurrentUsername( QString username ) = 0;
        virtual UserSettingsServiceInterface* currentUser() = 0;
        virtual UserSettingsServiceInterface* user( QString username ) = 0;
        virtual bool deleteUser( QString username ) = 0;
        virtual bool existingUser( QString username ) = 0;

        virtual QStringList allPlugins() = 0;
        virtual QString pluginVersion( QString pluginId ) = 0;
        virtual QString pluginPlayerPath( QString pluginId ) = 0;
        virtual void setPluginPlayerPath( QString pluginId, QString path ) = 0;

        virtual QStringList allMediaDevices() = 0;
        virtual QString MediaDeviceUser( const QString& uid ) = 0;
        virtual void addMediaDevice( const QString& uid, const QString& username ) = 0;

        virtual int volume() = 0;
        virtual void setVolume( int volume ) = 0;
        virtual int soundCard() = 0;
        virtual void setSoundCard( int card ) = 0;

        virtual int soundSystem() = 0;
        virtual void setSoundSystem( int system ) = 0;
        virtual int externalSoundSystem() = 0;

        virtual QString proxyUsername() = 0;
        virtual void setProxyUsername( QString username ) = 0;
        virtual QString proxyPassword() = 0;
        virtual void setProxyPassword( QString password ) = 0;

        virtual QString proxyHost() = 0;
        virtual void setProxyHost( QString host ) = 0;
        virtual int proxyPort() = 0;
        virtual void setProxyPort( int port ) = 0;

        virtual bool useProxy() = 0;
        virtual void setUseProxy( bool enabled ) = 0;

        virtual QString browser() = 0;
        virtual void setBrowser( QString browser ) = 0;

        virtual bool firstRun() = 0;
        virtual void firstRunDone() = 0;
        virtual bool bootstrap() = 0;
        virtual void bootstrapDone() = 0;

        virtual void setDontAsk( const QString op, bool value ) = 0;
        virtual bool dontAsk( const QString op ) = 0;

        virtual void setAppLanguage( QString langCode ) = 0;
        virtual QString appLanguage() = 0; 

        virtual UserIconColour getFreeColour() = 0;

    public slots:
        virtual void save( bool restartConnection = false, bool restartAudio = false ) = 0;

    signals:
        virtual void reconnect() = 0;
        virtual void resetAudio() = 0;
        virtual void currentUserChanged( UserSettingsServiceInterface* user ) = 0;
};

Q_DECLARE_INTERFACE( SettingsServiceInterface, "fm.last.Service.Settings/1.0" )

#endif
