/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef SIDEBAR_EXTENSION_H
#define SIDEBAR_EXTENSION_H

#include <QObject>
#include <QFrame>

#include <extensioninterface.h>
#include <inputinterface.h>
#include <settingsserviceinterface.h>
#include <webserviceinterface.h>

#include "ui_sidebar.h"

static const int kTypeRole = Qt::UserRole;
static const int kArtistRole = Qt::UserRole + 1;
static const int kTrackRole = Qt::UserRole + 2;
static const int kWeightRole = Qt::UserRole + 3;
static const int kStationRole = Qt::UserRole + 10;
static const int kItemTypeRole = Qt::UserRole + 11;

class SideBarDelegate;
class SideBarTree;

class SidebarExtension : public ExtensionInterface
{
    Q_OBJECT
    Q_INTERFACES( ExtensionInterface )

    public:
        enum NodeType { PersonalRadio, NeighbourRadio, LovedRadio, RecommendedRadio, RecentlyPlayed, RecentlyLoved, RecentlyBanned,
                        Tags, Friends, Neighbours, Bookmarks, History, Station, Group, User, Tag, Artist, Album, Track, UserInfo };

        SidebarExtension();
        ~SidebarExtension();

        QString name() const;
        QString version() const;
        QString tabCaption() const { return tr( "Sidebar" ); }

        virtual bool hasGui() { return true; }

        QWidget* gui();
        bool guiEnabled() { return ( m_widget != 0 ); }

        QWidget* owner() { return m_parent; }
        void setOwner( QWidget* parent ) { m_parent = parent; setParent( parent ); }

        SettingsServiceInterface *m_iSettings;
        WebServiceInterface *m_iWebService;
        InputInterface *m_iInput;

    signals:
        void showFriendsTab();
        void stationFocused( const QString& url );
        void statusMessage( const QString& message );

    private:
        void initGui();

        Ui::SidebarExtension ui;
        QWidget *m_widget;
        QWidget *m_parent;

        QStandardItemModel *m_sidebarModel;
        SideBarDelegate *m_delegate;

    private slots:
        void addListenedTrack( const QString& artist, const QString& track );
        void addLovedTrack( const QString& artist, const QString& track );
        void addBannedTrack( const QString& artist, const QString& track );

        void removeListenedTrack( const QString& artist, const QString& track );
        void removeLovedTrack( const QString& artist, const QString& track );
        void removeBannedTrack( const QString& artist, const QString& track );
        void removeFriend( const QString& username );

        void updateStations();
        void updateFriends( const QString& username, const QStringList& friends );
        void updateNeighbours( const QString& username, const QList<QPair<QString, int> >& neighbours );
        void updateUserTags( const QString& username, const QList<QPair<QString, int> >& tags );
        void updateRecentTracks( const QString& username, QList< QPair<QString, QString> > songs );
        void updateRecentLovedTracks( const QString& username, QList< QPair<QString, QString> > songs );
        void updateRecentBannedTracks( const QString& username, QList< QPair<QString, QString> > songs );

        void updateHistory();
        void updateAll();

        void currentChanged( const QModelIndex& index );
        void doubleClicked( const QModelIndex& index );
        void stationChanged( QString url, QString name );
        void stationName( QString url, QString name );
};

#endif
