/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include "sidebartree.h"
#include "sidebarextension.h"
#include "treestyle.h"

#include <confirmdialog.h>
#include <containerutils.h>

#include "Loqqer.h"

#ifdef Q_WS_MAC
    static const int k_fontSize = 11;
#else
    static const int k_fontSize = 11;
#endif

static const QColor k_milkyBlue = QColor( 0xa4, 0xb4, 0xcb );
//( 0x98, 0xaa, 0xc4 );

SideBarDelegate::SideBarDelegate( QObject *parent )
    : QItemDelegate( parent )
{
}


QSize
SideBarDelegate::sizeHint( const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
    QString type = index.data( kTypeRole ).toString();
    bool toplevel = ( !index.parent().isValid() && type != "station" );

    if ( type == "userinfo" )
    {
        #ifdef Q_WS_MAC
            return QSize( 0, 51 );
        #else
            return QSize( 0, 38 );
        #endif
    }
    else
    if ( type == "spacer" )
    {
        #ifndef Q_WS_MAC
            return QSize( 0, 0 );
        #else
            return QSize( 0, 8 );
        #endif
    }
    else
    {
        #ifndef Q_WS_MAC
            if ( toplevel || type == "station" )
            {
                return QSize( 0, 20 );
            }
            else
            {
                return QSize( 0, 17 );
            }
        #else
            if ( toplevel )
            {
                return QSize( 0, 28 );
            }
            else
            {
                return QSize( 0, 20 );
            }
        #endif
    }
}


void
SideBarDelegate::drawFocus( QPainter *painter,const QStyleOptionViewItem & option, const QRect & rect ) const
{
}


void
SideBarDelegate::paint( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
    bool selected = ( option.state & QStyle::State_Selected );
    QString type = index.data( kTypeRole ).toString();

    QFont font = painter->font();
    font.setPixelSize( k_fontSize );
    painter->setFont( font );

    bool toplevel = ( !index.parent().isValid() && type != "station" );
    if ( type == "spacer" )
    {
        // pretty empty here :)
    }
    else

    #ifndef Q_WS_MAC
        QItemDelegate::paint( painter, option, index );
    #else

        if ( toplevel )
        {
            painter->setViewTransformEnabled( true );
            QRect rect = option.rect;
            rect.adjust( 0, 0, 0, 28 );
            if ( selected )
            {
                painter->setPen( QColor( Qt::white ) );
                painter->fillRect( rect, k_milkyBlue );
            }
            else
            {
                painter->setPen( QColor( 0x00, 0x00, 0x00 ) );
                #ifdef Q_WS_MAC
                    painter->fillRect( rect, QColor( 0xe5, 0xed, 0xf7 ) );
                #else
                    painter->fillRect( rect, QColor( 0xff, 0xff, 0xff ) );
                #endif
            }

            rect.adjust( 0, 2, 0, 0 );
            QIcon icon = index.data( Qt::DecorationRole ).value<QIcon>();
            painter->drawPixmap( rect.left(), rect.top(), 24, 24, icon.pixmap( 24, 24 ) );

            QTextOption noWrap;
            noWrap.setWrapMode( QTextOption::NoWrap );
            rect.adjust( 28, 5, 0, 0 );
            painter->drawText( rect, index.data( Qt::DisplayRole ).toString(), noWrap );
        }
        else if ( !toplevel )
        {
            painter->setViewTransformEnabled( true );
            QRect rect = option.rect;

            if ( index.parent().isValid() )
                rect.adjust( -20, 0, 0, 20 );

            if ( selected )
            {
                painter->setPen( QColor( 0xff, 0xff, 0xff ) );
                painter->fillRect( rect, k_milkyBlue );
            }
            else
            {
                painter->setPen( QColor( 0x00, 0x00, 0x00 ) );
                #ifdef Q_WS_MAC
                    painter->fillRect( rect, QColor( 0xe5, 0xed, 0xf7 ) );
                #else
                    painter->fillRect( rect, QColor( 0xff, 0xff, 0xff ) );
                #endif
            }

            rect.adjust( 0, 2, 0, 0 );
            QIcon icon = index.data( Qt::DecorationRole ).value<QIcon>();
            painter->drawPixmap( rect.left() + 8, rect.top(), 16, 16, icon.pixmap( 16, 16 ) );

            rect.adjust( 28, 0, 0, 0 );
            QTextOption noWrap;
            noWrap.setWrapMode( QTextOption::NoWrap );
            painter->drawText( rect, index.data( Qt::DisplayRole ).toString(), noWrap );
        }
        else
            QItemDelegate::paint( painter, option, index );
    #endif // Q_WS_MAC
}


SidebarTree::SidebarTree( QWidget* parent ) :
    QTreeView( parent )
{
    m_iWebService = webService();
    m_iInput = inputService();
    setEditTriggers( QAbstractItemView::NoEditTriggers );
    setAnimated( false );

    #ifndef Q_WS_MAC
    setStyle( new TreeStyle() );
    #endif

    m_timer.setInterval( 500 );
    connect( &m_timer, SIGNAL( timeout() ), this, SLOT( expandCurrentItem() ) );

    connect( m_iWebService, SIGNAL( userPicsResult( const QMap<QString, QString>&, bool ) ),
             this,          SLOT( userPicsFinished( const QMap<QString, QString>&, bool ) ) );

    connect( settingsService(), SIGNAL( userSwitch( UserSettingsServiceInterface* ) ),
             this,              SLOT( setUser( UserSettingsServiceInterface* ) ) );
}

void
SidebarTree::setUser( UserSettingsServiceInterface* user )
{
    // HACK! Commenting this optimisation out fixes the intermittent 
    // disappearance of the user avatar as the first time it comes on here
    // after a login, the call the userPics will fail due to no basehost
    // having been set yet. The second time it gets here all is well though.
    //if ( user->username() == m_userName )
    //{
    //    return;
    //}

    m_userName = user->username();

    QStringList sl;
    sl << m_userName;
    m_iWebService->userPics( sl );

    viewport()->update();
}

void
SidebarTree::drawRow( QPainter *painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
    bool selected = ( currentIndex() == index );
    QString type = index.data( kTypeRole ).toString();

    QFont font = painter->font();
    #ifdef Q_WS_MAC
        font.setPixelSize( k_fontSize + 1 );
    #else
        font.setPixelSize( k_fontSize );
    #endif
    painter->setFont( font );

    if ( type == "userinfo" )
    {
        painter->setViewTransformEnabled( true );

        QRect rect = option.rect;
        rect.adjust( 0, 0, width(), 0 );

        painter->setPen( QColor( 0xff, 0xff, 0xff ) );
        QLinearGradient gradient( 0, 0, 0, rect.height() );
        gradient.setColorAt( 0, QColor( 0xab, 0xb9, 0xcf ) );
        gradient.setColorAt( 1, QColor( 0x99, 0xab, 0xc4 ) );
        painter->fillRect( rect, QBrush( gradient ) );

        if ( !m_userPic.isNull() )
            painter->drawPixmap(
                QPoint( 12 + ( 30 - m_userPic.width() ) / 2, ( rect.height() - 30 ) / 2 ),
                m_userPic );

        QTextOption noWrap;
        noWrap.setWrapMode( QTextOption::NoWrap );

		#ifdef Q_WS_MAC
        	rect.adjust( 48, 10, 0, -10 );
		#else
			rect.adjust( 48, 6, 0, -6 );
		#endif
        painter->drawText( rect, tr( "My Profile" ), noWrap );

        rect.adjust( 0, painter->fontMetrics().height() - 2, 0, 0 );
        painter->drawText( rect, m_userName, noWrap );
    }
    else if ( type != "spacer" )
        QTreeView::drawRow( painter, option, index );
}


void
SidebarTree::drawBranches( QPainter* painter, const QRect& rect, const QModelIndex& index ) const
{
    QString type = index.data( kTypeRole ).toString();

    QRect r = rect;
    #ifdef Q_WS_MAC
        r.translate( 4, 0 );
    #endif

    QTreeView::drawBranches( painter, r, index );
}


void
SidebarTree::deleteFriend()
{
    if ( currentIndex().isValid() )
    {
        int confirm = QMessageBox::question( this, tr( "Confirm" ),
                                             tr( "Do you really want to remove %1 from your friends list?" ).arg( currentIndex().data().toString() ),
                                             QMessageBox::Yes, QMessageBox::No );

        if ( confirm == QMessageBox::Yes )
            m_iWebService->deleteFriend( currentIndex().data().toString() );
    }
}


void
SidebarTree::unloveTrack()
{
    if ( currentIndex().isValid() )
    {
        int confirm = QMessageBox::question( this, tr( "Confirm" ),
                                             tr( "Do you really want to remove %1 from your Loved Tracks?" ).arg( currentIndex().data().toString() ),
                                             QMessageBox::Yes, QMessageBox::No );

        if ( confirm == QMessageBox::Yes )
            m_iWebService->unlove( currentIndex().data( kArtistRole ).toString(),
                                   currentIndex().data( kTrackRole ).toString() );
    }
}


void
SidebarTree::unbanTrack()
{
    if ( currentIndex().isValid() )
    {
        int confirm = QMessageBox::question( this, tr( "Confirm" ),
                                             tr( "Do you really want to un-ban %1?" ).arg( currentIndex().data().toString() ),
                                             QMessageBox::Yes, QMessageBox::No );

        if ( confirm == QMessageBox::Yes )
            m_iWebService->unban( currentIndex().data( kArtistRole ).toString(),
                                  currentIndex().data( kTrackRole ).toString() );
    }
}


void
SidebarTree::unlistenTrack()
{
    if ( currentIndex().isValid() )
    {
        int confirm = QMessageBox::question( this, tr( "Confirm" ),
                                             tr( "Do you really want to remove %1 from your profile?" ).arg( currentIndex().data().toString() ),
                                             QMessageBox::Yes, QMessageBox::No );

        if ( confirm == QMessageBox::Yes )
            m_iWebService->unlisten( currentIndex().data( kArtistRole ).toString(),
                                     currentIndex().data( kTrackRole ).toString() );
    }
}


void
SidebarTree::contextMenuEvent( QContextMenuEvent* event )
{
    QModelIndex index = indexAt( event->pos() );
    m_lastRightClickedIdx = index;
    if ( index.isValid() )
    {
        QMenu menu( this );

        int parentType = -1;
        int type = index.data( kItemTypeRole ).toInt();
        QString text = index.data().toString();

        if ( index.parent().isValid() )
            parentType = index.parent().data( kItemTypeRole ).toInt();

        QString playStation = tr( "Play this station" );

        // Mac GUI guidelines wants ellipsis after actions that always pop up
        // confirm dialogs.
        #ifdef Q_WS_MAC
        QString ellipsis = tr( "..." );
        #else
        QString ellipsis = "";
        #endif

        switch( type )
        {
            case SidebarExtension::PersonalRadio:
            {
                QIcon icon;
                #ifndef Q_WS_MAC
                    icon.addFile( dataPath( "icons/personal_radio.png" ) );
                #endif
                menu.addAction( icon, playStation,
                    this, SLOT( playMyPersonalRadio() ) );
                break;
            }

            case SidebarExtension::NeighbourRadio:
            {
                QIcon icon;
                #ifndef Q_WS_MAC
                    icon.addFile( dataPath( "icons/neighbour_radio.png" ) );
                #endif
                menu.addAction( icon, playStation,
                    this, SLOT( playMyNeighbourRadio() ) );
                break;
            }

            case SidebarExtension::LovedRadio:
            {
                QIcon icon;
                #ifndef Q_WS_MAC
                    icon.addFile( dataPath( "icons/loved_radio.png" ) );
                #endif
                menu.addAction( icon, playStation,
                    this, SLOT( playMyLovedRadio() ) );
                break;
            }

            case SidebarExtension::RecommendedRadio:
            {
                QIcon icon;
                #ifndef Q_WS_MAC
                    icon.addFile( dataPath( "icons/recommended_radio.png" ) );
                #endif
                menu.addAction( icon, playStation,
                    this, SLOT( playMyRecommendedRadio() ) );
                break;
            }

            case SidebarExtension::Tag:
            {
            #ifndef HIDE_RADIO
                QIcon icon;
                #ifndef Q_WS_MAC
                    icon.addFile( dataPath( "icons/my_tags.png" ) );
                #endif
                menu.addAction( icon, tr( "Play this tag station" ),
                    this, SLOT( playGlobalTagRadio() ) );
                menu.addAction( icon, tr( "Play only music you tagged" ).arg( text ),
                    this, SLOT( playUserTagRadio() ) );
            #endif // HIDE_RADIO
                break;
            }

            case SidebarExtension::User:
            {
            #ifndef HIDE_RADIO
                QIcon icon;
                #ifndef Q_WS_MAC
                    icon = QIcon( dataPath( "icons/personal_radio.png" ) );
                #endif
                menu.addAction( icon, tr( "Play %1's Personal Radio" ).arg( text ),
                    this, SLOT( playPersonalRadio() ) );
                #ifndef Q_WS_MAC
                    icon = QIcon( dataPath( "icons/neighbour_radio.png" ) );
                #endif
                menu.addAction( icon, tr( "Play %1's Neighbourhood Radio" ).arg( text ),
                    this, SLOT( playNeighbourRadio() ) );
                #ifndef Q_WS_MAC
                    icon = QIcon( dataPath( "icons/loved_radio.png" ) );
                #endif
                menu.addAction( icon, tr( "Play %1's Loved Tracks Radio" ).arg( text ),
                    this, SLOT( playLovedRadio() ) );
            #endif // HIDE_RADIO

                if ( parentType == SidebarExtension::Friends )
                {
                #ifndef HIDE_RADIO
                    menu.addSeparator();
                #endif // HIDE_RADIO
                    menu.addAction( QIcon(), tr( "End friendship%1" ).arg( ellipsis ),
                        this, SLOT( deleteFriend() ) );
                }
                break;
            }

            case SidebarExtension::Track:
            {

                if ( parentType == SidebarExtension::RecentlyPlayed )
                {
                    menu.addAction( QIcon(), tr( "Remove from profile%1" ).arg( ellipsis ),
                        this, SLOT( unlistenTrack() ) );
                }

                if ( parentType == SidebarExtension::RecentlyLoved )
                {
                    menu.addAction( QIcon(), tr( "Remove from your Loved Tracks%1" ).arg( ellipsis ),
                        this, SLOT( unloveTrack() ) );
                }

                if ( parentType == SidebarExtension::RecentlyBanned )
                {
                    menu.addAction( QIcon(), tr( "Un-ban this track%1" ).arg( ellipsis ),
                        this, SLOT( unbanTrack() ) );
                }
                break;
            }

            case SidebarExtension::History:
                menu.addAction( QIcon(), tr( "Empty your station history" ),
                    this, SLOT( clearHistory() ) );
                break;

            case SidebarExtension::Station:
            {
                QIcon icon;
                #ifndef Q_WS_MAC
                    icon.addFile( dataPath( "icons/icon_radio.png" ) );
                #endif
                menu.addAction( icon, playStation,
                    this, SLOT( startStationFromContextMenu() ) );
            }
            break;
            
            default:
                return;
        }

        menu.exec( event->globalPos() );
    }
}


void
SidebarTree::mousePressEvent( QMouseEvent *event )
{
    QTreeView::mousePressEvent( event );

    if ( event->button() == Qt::LeftButton )
        m_dragStartPosition = event->pos();
}


void
SidebarTree::mouseReleaseEvent( QMouseEvent *event )
{
    QTreeView::mouseReleaseEvent( event );
}


void
SidebarTree::mouseMoveEvent( QMouseEvent *event )
{
    QTreeView::mouseMoveEvent( event );

    if ( !( event->buttons() & Qt::LeftButton ) || !currentIndex().isValid() )
        return;
    if ( ( event->pos() - m_dragStartPosition ).manhattanLength() < QApplication::startDragDistance() )
        return;

    QString anchor = currentIndex().data().toString();
    if ( !anchor.isEmpty() )
    {
        QDrag *drag = new QDrag( this );
        QMimeData *mimeData = new QMimeData();
        mimeData->setText( anchor );

        int itemType;
        qDebug() << currentIndex().data( kItemTypeRole );
        switch( currentIndex().data( kItemTypeRole ).toInt() )
        {
            case SidebarExtension::Artist:
                itemType = ItemArtist;
                break;

            case SidebarExtension::Album:
                itemType = ItemAlbum;
                break;

            case SidebarExtension::Track:
                itemType = ItemTrack;
                break;

            case SidebarExtension::Tag:
                itemType = ItemTag;
                break;

            case SidebarExtension::User:
                itemType = ItemUser;
                break;

            case SidebarExtension::Station:
                itemType = ItemStation;
                break;

            default:
                return;
        }

        mimeData->setData( "item/type", QByteArray::number( itemType ) );

        QHash<QString, QString> data;
        switch( itemType )
        {
            case ItemArtist:
                mimeData->setData( "item/artist", currentIndex().data( kArtistRole ).toByteArray() );
                break;

            case ItemAlbum:
                mimeData->setData( "item/artist", currentIndex().data( kArtistRole ).toByteArray() );
                mimeData->setData( "item/album", currentIndex().data( kTrackRole ).toByteArray() );
                break;

            case ItemTrack:
                mimeData->setData( "item/artist", currentIndex().data( kArtistRole ).toByteArray() );
                mimeData->setData( "item/track", currentIndex().data( kTrackRole ).toByteArray() );
                break;

            case ItemTag:
                mimeData->setData( "item/tag", currentIndex().data( kArtistRole ).toByteArray() );
                break;

            case ItemUser:
                mimeData->setData( "item/user", currentIndex().data( kArtistRole ).toByteArray() );
                break;

            case ItemStation:
                mimeData->setData( "item/station", anchor.toUtf8() );
                break;
        }

        QPainter painter;
        QPixmap pixmap( painter.fontMetrics().width( anchor ) + 16, painter.fontMetrics().height() + 4 );
        QRect rect( 0, 0, pixmap.width() - 1, pixmap.height() - 1 );

        painter.begin( &pixmap );
        painter.setBackgroundMode( Qt::OpaqueMode );

        QFont font = painter.font();
        font.setPixelSize( k_fontSize );
        painter.setFont( font );

        painter.setBrush( Qt::white );
        painter.setPen( Qt::black );
        painter.drawRect( rect );

        painter.setPen( Qt::black );
        painter.drawText( rect, Qt::AlignCenter, anchor );
        painter.end();

        drag->setMimeData( mimeData );
        drag->setPixmap( pixmap );

        Qt::DropAction dropAction = drag->start( Qt::CopyAction );
    }
}


void
SidebarTree::dragEnterEvent( QDragEnterEvent* event )
{
    QTreeView::dragEnterEvent( event );
    event->acceptProposedAction();
}


void
SidebarTree::dragMoveEvent( QDragMoveEvent* event )
{
    QTreeView::dragMoveEvent( event );
    QModelIndex index = indexAt( event->pos() );
    if ( !index.isValid() )
    {
        event->ignore();
        return;
    }

    setCurrentIndex( index );
    m_timer.stop();
    m_timer.start();

    QString statusToken;
    int itemType = event->mimeData()->data( "item/type" ).toInt();
    if ( itemType == ItemAlbum )
    {
        QString artist = QString::fromUtf8( event->mimeData()->data( "item/artist" ) );
        QString album = QString::fromUtf8( event->mimeData()->data( "item/album" ) );
        statusToken = artist + " - " + album;
    }
    else if ( itemType == ItemTrack )
    {
        QString artist = QString::fromUtf8( event->mimeData()->data( "item/artist" ) );
        QString track = QString::fromUtf8( event->mimeData()->data( "item/track" ) );
        statusToken = artist + " - " + track;
    }
    else if ( itemType == ItemArtist )
    {
        QString artist = QString::fromUtf8( event->mimeData()->data( "item/artist" ) );
        statusToken = artist;
    }

    bool accept = false;
    switch( index.data( kItemTypeRole ).toInt() )
    {
        case SidebarExtension::User:
        {
            if ( itemType == ItemArtist || itemType == ItemAlbum || itemType == ItemTrack )
            {
                emit statusMessage( tr( "Recommends %1 to %2" ).arg( statusToken ).arg( index.data().toString() ) );
                accept = true;
            }
            break;
        }

        case SidebarExtension::Tag:
            if ( itemType == ItemArtist || itemType == ItemAlbum || itemType == ItemTrack )
            {
                emit statusMessage( tr( "Tags %1 as %2" ).arg( statusToken ).arg( index.data().toString() ) );
                accept = true;
            }
            break;
    }

    if ( accept )
    {
        event->accept();
//         event->accept( visualItemRect( item ) );
    }
    else
        event->ignore();
}


void
SidebarTree::dropEvent( QDropEvent* event )
{
    m_timer.stop();

    QModelIndex index = indexAt( event->pos() );
    if ( !index.isValid() )
        return;

    int itemType = event->mimeData()->data( "item/type" ).toInt();
    switch( itemType )
    {
        case ItemArtist:
            break;

        case ItemAlbum:
            break;

        case ItemTrack:
            break;

        case ItemTag:
            break;

        case ItemUser:
            break;

        default:
            
            // OK Chris, I know you hate asserts but why are you doing this? If
            // we get an unknown drop surely it's a logic error somewhere and we
            // want to be told about it. Go asserts go! :)

            // unknown, this shouldn't happen!
            qDebug() << "Unknown drop!";
            event->ignore();
            return;
    }

    switch( index.data( kItemTypeRole ).toInt() )
    {
        case SidebarExtension::User:
        {
            QString message;
            QString artist;
            QString token;

            switch( itemType )
            {
                case ItemArtist:
                    artist = QString::fromUtf8( event->mimeData()->data( "item/artist" ) );
                    token = "";
                    break;

                case ItemAlbum:
                    artist = QString::fromUtf8( event->mimeData()->data( "item/artist" ) );
                    token = QString::fromUtf8( event->mimeData()->data( "item/album" ) );
                    break;

                case ItemTrack:
                    artist = QString::fromUtf8( event->mimeData()->data( "item/artist" ) );
                    token = QString::fromUtf8( event->mimeData()->data( "item/track" ) );
                    break;

            }

            QString confirmMsg = QString( tr( "Are you sure you want to recommend %1 to %2?" ) )
                .arg( token.isEmpty() ? artist : token )
                .arg( index.data().toString() );

            ConfirmDialog dlg( ConfirmDialog::Recommend, confirmMsg, this );
            if ( dlg.exec() == QDialog::Accepted )
            {
                m_iWebService->recommend( itemType, index.data().toString(), message, artist, token );
            }
            break;
        }

        case SidebarExtension::Tag:
        {
            QString artist;
            QString token;

            switch( itemType )
            {
                case ItemArtist:
                    artist = QString::fromUtf8( event->mimeData()->data( "item/artist" ) );
                    token = "";
                    break;

                case ItemAlbum:
                    artist = QString::fromUtf8( event->mimeData()->data( "item/artist" ) );
                    token = QString::fromUtf8( event->mimeData()->data( "item/album" ) );
                    break;

                case ItemTrack:
                    artist = QString::fromUtf8( event->mimeData()->data( "item/artist" ) );
                    token = QString::fromUtf8( event->mimeData()->data( "item/track" ) );
                    break;
            }

            QString confirmMsg = QString( tr( "Are you sure you want to tag %1 as %2?" ) )
                .arg( token.isEmpty() ? artist : token )
                .arg( index.data().toString() );

            ConfirmDialog dlg( ConfirmDialog::Tag, confirmMsg, this );
            if ( dlg.exec() == QDialog::Accepted )
            {
                m_iWebService->tagItem( itemType, TAG_APPEND, index.data().toString(), artist, token );
            }
            break;
        }
    }

    event->accept();
}

void
SidebarTree::userPicsFinished( const QMap<QString, QString>& picUrls, bool error )
{
    if ( error || picUrls.isEmpty() )
    {
        LOGL( 1, "User pic request failed" );
        return;
    }
    
    QString url = picUrls[ m_userName.toLower() ];
    
    if ( url.isEmpty() )
    {
        LOGL( 1, "User pic url empty" );
        return;
    }
    
    downloadPic( url );
}


// These so don't belong here but this needs some major refactoring anyway :(
void
SidebarTree::downloadPic( const QString& picUrl )
{
    QUrl url( picUrl );
    Http* http = new Http();
    http->setHost( url.host() );
 
    if ( !url.encodedQuery().isEmpty() )
    {
        http->get( url.path() + "?" + url.encodedQuery(), false );
    }
    else
    {
        http->get( url.path(), false );
    }

    connect( http, SIGNAL( dataAvailable( QByteArray ) ),
             this,   SLOT( downloadPicFinished( QByteArray ) ) );
}

void
SidebarTree::downloadPicFinished( const QByteArray& buffer )
{
    if ( buffer.size() <= 0 )
    {
        LOGL( 1, "User pic buffer == 0" );
        return;
    }

    bool success = m_userPic.loadFromData( buffer );

    if ( success )
    {
        m_userPic = m_userPic.scaled( 30, 30, Qt::KeepAspectRatio, Qt::SmoothTransformation );
        viewport()->update();
    }
    else
    {
        LOGL( 1, "Loading of image from byte array failed." );
    }

    sender()->deleteLater();
}


void
SidebarTree::clearHistory()
{
    settingsService()->currentUser()->clearRecentStations( true );
}

void
SidebarTree::startStationFromContextMenu()
{
    QString station = m_lastRightClickedIdx.data( kStationRole ).toString();
    emit stationRequested( station );
}
