/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef SIDEBAR_TREE_H
#define SIDEBAR_TREE_H

#include <QtGui>

#include "inputinterface.h"
#include "webserviceinterface.h"

class UserSettingsServiceInterface;

class SideBarDelegate : public QItemDelegate
{
    Q_OBJECT

    public:
        SideBarDelegate( QObject *parent = 0 );

        void drawFocus( QPainter *painter,const QStyleOptionViewItem & option, const QRect & rect ) const;
        void paint( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const;
        QSize sizeHint( const QStyleOptionViewItem & option, const QModelIndex & index ) const;
};


class SidebarTree : public QTreeView
{
    Q_OBJECT

    public:
        SidebarTree( QWidget* parent = 0 );
        void setWebService( WebServiceInterface* ws ) { m_iWebService = ws; }
        void setInputService( InputInterface* input ) { m_iInput = input; }

    public slots:
        void setUser( UserSettingsServiceInterface* userName );

    signals:
        void statusMessage( const QString& message );
        void stationRequested( QString stationUrl );

    private slots:
        void expandCurrentItem() { if ( currentIndex().isValid() && currentIndex().model()->hasChildren( currentIndex()) ) expand( currentIndex() ); }

        // REFACTOR. Duplicated functionality. Sidebarextension already knows how to do this stuff in its doubleClicked function.
        // Replace all these with calls to the startStationFromContextMenu slot instead.

        void playPersonalRadio() { m_iInput->load( QString( "lastfm://user/%1/personal" ).arg( currentIndex().data().toString() ) ); }
        void playNeighbourRadio() { m_iInput->load( QString( "lastfm://user/%1/neighbours" ).arg( currentIndex().data().toString() ) ); }
        void playLovedRadio() { m_iInput->load( QString( "lastfm://user/%1/loved" ).arg( currentIndex().data().toString() ) ); }

        void playMyPersonalRadio() { m_iInput->load( QString( "lastfm://user/%1/personal" ).arg( m_iWebService->currentUsername() ) ); }
        void playMyNeighbourRadio() { m_iInput->load( QString( "lastfm://user/%1/neighbours" ).arg( m_iWebService->currentUsername() ) ); }
        void playMyLovedRadio() { m_iInput->load( QString( "lastfm://user/%1/loved" ).arg( m_iWebService->currentUsername() ) ); }
        void playMyRecommendedRadio() { m_iInput->load( QString( "lastfm://user/%1/recommended" ).arg( m_iWebService->currentUsername() ) ); }

        void playGlobalTagRadio() { m_iInput->load( QString( "lastfm://globaltags/%1" ).arg( currentIndex().data().toString() ) ); }
        void playUserTagRadio() { m_iInput->load( QString( "lastfm://usertags/%1/%2" ).arg( m_iWebService->currentUsername() ).arg( currentIndex().data().toString() ) ); }

        void startStationFromContextMenu();

        void deleteFriend();
        void unloveTrack();
        void unbanTrack();
        void unlistenTrack();
        void clearHistory();

        void downloadPicFinished( const QByteArray& buffer );
        void userPicsFinished( const QMap<QString, QString>& picUrls, bool error );

    protected:
        void contextMenuEvent( QContextMenuEvent* event );

        void mousePressEvent( QMouseEvent *event );
        void mouseReleaseEvent( QMouseEvent *event );
        void mouseMoveEvent( QMouseEvent *event );

        void dropEvent( QDropEvent* event );
        void dragEnterEvent( QDragEnterEvent* event );
        void dragLeaveEvent( QDragLeaveEvent* event ) { m_timer.stop(); }
        void dragMoveEvent( QDragMoveEvent* event );

        void drawRow( QPainter *painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const;
        void drawBranches( QPainter* painter, const QRect& rect, const QModelIndex& index ) const;

    private:
        void downloadPic( const QString& url );

        WebServiceInterface* m_iWebService;
        InputInterface* m_iInput;

        QTimer m_timer;
        QPoint m_dragStartPosition;

        QString m_userName;
        QPixmap m_userPic;
        
        QModelIndex m_lastRightClickedIdx;
};


class BigTreeItem : public QTreeWidgetItem
{
    public:
        BigTreeItem( QTreeWidget* parent = 0, int type = 0 ) :
            QTreeWidgetItem( parent, type )
        {
            #ifndef WIN32
            setSizeHint( 0, QSize( 0, 26 ) );
            #else
            setSizeHint( 0, QSize( 0, 20 ) );
            #endif
        }
};

#endif
