/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include <QtGui>

#include "skypenotifyextension.h"
#include <containerutils.h>
#include "Loqqer.h"

#define SKYPE_SEND_URL (0)

SkypeNotifyExtension::SkypeNotifyExtension() :
    m_parent( NULL ),
    m_settingsPanel( NULL ),
    m_defaultEnabled( false ),
    m_defaultFormat( tr( "Currently listening to '%2' by %1" ) )
{
    gLogger.Init( savePath( "skype.log" ), false );
    gLogger.SetLevel( 4 );
    
    LOG( 3, "Initialising Skype Extension\n" );
}


QString
SkypeNotifyExtension::name() const
{
    return QString( "Skype Notifier Extension" );
}


QString
SkypeNotifyExtension::version() const
{
    return QString( "1.0.0.0" );
}

QWidget*
SkypeNotifyExtension::settingsPane()
{
    if ( !m_settingsPanel )
    {
        initSkypeCommunication();
        initSettingsPanel();
    }

    return m_settingsPanel;
}

QPixmap*
SkypeNotifyExtension::settingsIcon()
{
    Q_ASSERT( !m_icon.isNull() );

    return &m_icon;
}

void
SkypeNotifyExtension::populateSettings()
{
    // This won't get called until the options pane is displayed for 
    // the first time.

    QSettings* config = settingsService()->currentUser()->qSettings();

    config->beginGroup( name() );
    ui.enabledCheck->setChecked(
        config->value( "Enabled", m_defaultEnabled ).toBool() );
    ui.nowPlayingFormat->setText(
        config->value( "Format", m_defaultFormat ).toString() );
    config->endGroup();
}

void
SkypeNotifyExtension::saveSettings()
{
    QSettings* config = settingsService()->currentUser()->qSettings();

    if ( !ui.enabledCheck->isChecked() )
    {
        notify( MetaData() );
    }

    config->beginGroup( name() );
    config->setValue( "Enabled", ui.enabledCheck->isChecked() );
    config->setValue( "Format", ui.nowPlayingFormat->text() );
    config->endGroup();
}

void
SkypeNotifyExtension::notify( MetaData metaData )
{
    if ( !isSkypeAvailable() || !isEnabled() ) { return; }
    
    QString cmd = "SET PROFILE MOOD_TEXT ";
    
    QByteArray utfToSend;
    if ( metaData.isEmpty() )
    {
        utfToSend = cmd.toUtf8();
    }
    else
    {
        QString format = nowPlayingFormat();
        QString nowPlaying = cmd + format.arg( metaData.artist(),
                                               metaData.track(),
                                               metaData.album() );

        #if SKYPE_SEND_URL
            QString url = metaData.trackPageUrl();
            if ( !url.isEmpty() )
            {
                if ( url.startsWith( "http://" ) ) { url.remove( 0, 7 ); }
                if ( url.endsWith( "/" ) ) { url.chop( 1 ); }
                url.append( "/?s" ); // identifies link as coming from Skype

                nowPlaying += " " + url;
            }
        #endif // SKYPE_SEND_URL                
        
        utfToSend = nowPlaying.toUtf8();
    }

    sendToSkype( utfToSend );
}

bool
SkypeNotifyExtension::isEnabled()
{
    QSettings* config = settingsService()->currentUser()->qSettings();
    return config->value( name() + "/Enabled", m_defaultEnabled ).toBool();
}

QString
SkypeNotifyExtension::nowPlayingFormat()
{
    QSettings* config = settingsService()->currentUser()->qSettings();
    return config->value( name() + "/Format", m_defaultFormat ).toString();
}

void
SkypeNotifyExtension::initSettingsPanel()
{
    LOG( 3, "Initialising Skype GUI\n" );

    m_settingsPanel = new QWidget( m_parent );
    ui.setupUi( m_settingsPanel );

    // Loaded from resources to avoid having to ship a separate file
    bool t = m_icon.load( ":/options_skype.png" );
    
    connect( ui.enabledCheck,     SIGNAL( stateChanged( int ) ),
             this,                SIGNAL( settingsChanged() ) );
    connect( ui.nowPlayingFormat, SIGNAL( textChanged( QString ) ),
             this,                SIGNAL( settingsChanged() ) );
}

