/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef STOPWATCH_H
#define STOPWATCH_H

#include <QtCore>

class CPlayerConnection;

/*************************************************************************/ /**
    A timer class that counts seconds. It spawns its own thread which just
    sits there and counts. On timeout it notifies whoever's registered with it.
******************************************************************************/
class CStopWatch : public QThread
{
    Q_OBJECT

public:

    /*********************************************************************/ /**
        Ctors
    **************************************************************************/
    CStopWatch();
    CStopWatch(
        const CStopWatch& that);
    CStopWatch& operator=(
        const CStopWatch& that);

    /*********************************************************************/ /**
        Start counting. Will spawn thread.
    **************************************************************************/
    void
    Start(
        CPlayerConnection* pOwner);

    /*********************************************************************/ /**
        Stop timer.
    **************************************************************************/
    void
    Stop();

    /*********************************************************************/ /**
        Reset timer to zero.
    **************************************************************************/
    void
    Reset();

    /*********************************************************************/ /**
        Set the timeout.

        @param[in] nTimeout Timeout in seconds
    **************************************************************************/
    void
    SetTimeout(
        int nTimeout) { mnTimeout = nTimeout; emit timeoutChanged(mnTimeout); }

    /*********************************************************************/ /**
        Returns current time.
    **************************************************************************/
    int
    GetTime() { return mnTimer; }

    /*********************************************************************/ /**
        Returns the timeout.
    **************************************************************************/
    int
    GetTimeOut() { return mnTimeout; }

signals:

    /*********************************************************************/ /**
        Emitted when timer value changes.
    **************************************************************************/
    void
    valueChanged(
        int elapsed);

    /*********************************************************************/ /**
        Emitted when timer is reset.
    **************************************************************************/
    void
    timerReset();

    /*********************************************************************/ /**
        Emitted when timeout is changed.
    **************************************************************************/
    void
    timeoutChanged(
        int timeOut);


private:

    /*********************************************************************/ /**
        Used by copy ctor and operator=
    **************************************************************************/
    void
    Clone(
        const CStopWatch& that);

    enum EStopWatchState
    {
        STOPPED,
        RUNNING
    };

    /*********************************************************************/ /**
        QThread run method.
    **************************************************************************/
    virtual void
    run();

    EStopWatchState mState;

    int mnSubSecTimer;
    int mnTimer;
    int mnTimeout;

    CPlayerConnection* mpOwner;

    QMutex mMutex;
};

#endif // STOPWATCH_H
