/***************************************************************************
 *   Copyright (C) 2005 - 2006 by                                          *
 *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
 *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#ifndef SYSTRAY_H
#define SYSTRAY_H

#include "settingsserviceinterface.h"
#include "trayicon/trayicon.h"
#include "metadata.h"

#include <QtCore>
#include <QtGui>

class Container;

/*************************************************************************/ /**
    A thin wrapper around the TrayIcon class to deal with application-
    specific tray stuff.
******************************************************************************/
class SysTray : public QObject
{
    Q_OBJECT

    public:
        SysTray( Container* parent );

    public slots:
        void show() { m_trayIcon.show(); }

        void setUser( UserSettingsServiceInterface* currentUser );
        void setTrack( MetaData track );

        void setStatus( bool statusOK );
        void setToolTip( const QString& tooltip ) { m_trayIcon.setToolTip( tooltip ); }

    private:
        void refreshToolTip();

        #if QT_VERSION >= 0x040200
        QSystemTrayIcon   m_trayIcon;
        #else
        TrayIcon          m_trayIcon;
        #endif

        QMenu             m_trayMenu;
        QMenu*            m_userMenu;

        Container* m_parent;

        QString    m_tipFormat;
        QString    m_artist;
        QString    m_track;
        QString    m_user;

        bool       m_badStatus;

    private slots:
        void aboutToShowMenu();

        #if QT_VERSION >= 0x040200
        void trayAction( QSystemTrayIcon::ActivationReason reason );
        #endif
};

#endif // SYSTRAY_H
